@@ -3,13 +3,14 @@
  ** The Sleuth Kit 
  **
  ** Brian Carrier [carrier <at> sleuthkit [dot] org]
- ** Copyright (c) 2010-2013 Brian Carrier.  All Rights reserved
+ ** Copyright (c) 2010-2014 Brian Carrier.  All Rights reserved
  **
  ** This software is distributed under the Common Public License 1.0
  **
  */
 #include "tsk/tsk_tools_i.h"
 #include "tsk/auto/tsk_case_db.h"
+#include "tsk/hashdb/tsk_hash_info.h"
 #include "jni.h"
 #include "dataModel_SleuthkitJNI.h"
 #include <locale.h>
@@ -23,10 +24,8 @@ using std::string;
 using std::vector;
 using std::map;
 using std::stringstream;
-using std::for_each;
 
-static TSK_HDB_INFO * m_NSRLDb = NULL;
-static std::vector<TSK_HDB_INFO *> m_knownbads;
+static std::vector<TSK_HDB_INFO *> hashDbs;
 
 /*
 * JNI file handle structure encapsulates both
@@ -87,6 +86,7 @@ setThrowTskDataError(JNIEnv * env, const char *msg)
     env->ThrowNew(exception, msg);
 }
 
+#if 0
 /**
 * Sets flag to throw an TskDataException back up to the Java code with the currently set error message.
 * Note: exception is thrown to Java code after the native function returns
@@ -99,6 +99,7 @@ setThrowTskDataError(JNIEnv * env)
     const char *msg = tsk_error_get();
     setThrowTskDataError(env, msg);
 }
+#endif
 
 
 /***** Methods to cast from jlong to data type and check tags
@@ -175,15 +176,48 @@ castCaseDb(JNIEnv * env, jlong ptr)
     return lcl;
 }
 
+/**
+ * Convert a jstring (UTF-8) to a TCHAR to pass into TSK methods.
+ * @param buffer Buffer to store resulting string into
+ * @param size Length of buffer
+ * @param strJ string to convert
+ * @returns 1 on error 
+ */
 static int
 toTCHAR(JNIEnv * env, TSK_TCHAR * buffer, size_t size, jstring strJ)
 {
     jboolean isCopy;
     char *str8 = (char *) env->GetStringUTFChars(strJ, &isCopy);
 
-    int ret = TSNPRINTF(buffer, size, _TSK_T("%") PRIcTSK, str8);
+#ifdef TSK_WIN32
+	// Windows TCHAR is UTF16 in Windows, so convert
+    UTF16 *utf16 = (UTF16 *) buffer;
+    UTF8 *utf8 = (UTF8 *) str8;;
+    TSKConversionResult retval;
+    size_t lengthOfUtf8 = strlen(str8);
+
+    retval =
+        tsk_UTF8toUTF16((const UTF8 **) &utf8, &utf8[lengthOfUtf8],
+        &utf16, &utf16[size], TSKlenientConversion);
+    if (retval != TSKconversionOK) {
+        tsk_error_set_errno(TSK_ERR_IMG_CONVERT);
+        tsk_error_set_errstr
+            ("toTCHAR: Error converting UTF8 %s to UTF16, error %d",
+            utf8, retval);
+        env->ReleaseStringUTFChars(strJ, str8);
+        return 1;
+    }
+
+	// "utf16" now points to last char. Need to NULL terminate the string.
+    *utf16 = '\0';
+
+#else
+	// nothing to convert.  Keep it as UTF8
+	strncpy((char *)&buffer[0], str8, size);
+#endif
+
     env->ReleaseStringUTFChars(strJ, str8);
-    return ret;
+    return 0;
 }
 
 
@@ -199,7 +233,9 @@ JNIEXPORT jlong JNICALL
     jclass obj, jstring dbPathJ) {
 
     TSK_TCHAR dbPathT[1024];
-    toTCHAR(env, dbPathT, 1024, dbPathJ);
+    int retval = toTCHAR(env, dbPathT, 1024, dbPathJ);
+    if (retval)
+        return retval;
 
     TskCaseDb *tskCase = TskCaseDb::newDb(dbPathT);
 
@@ -256,180 +292,532 @@ JNIEXPORT void JNICALL
     return;
 }
 
-/*
- * Set the NSRL database to use for hash lookups.
- * @param env pointer to java environment this was called from
- * @param obj the java object this was called from
- * @param pathJ the path to the database
- * @return a handle for the nsrl database
+/**
+ * Opens an existing hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param pathJ The path to the hash database.
+ * @return A handle for the hash database.
  */
 JNIEXPORT jint JNICALL
-    Java_org_sleuthkit_datamodel_SleuthkitJNI_setDbNSRLNat(JNIEnv * env,
-    jclass obj, jstring pathJ) {
- 
-    if (m_NSRLDb != NULL) {
-        tsk_hdb_close(m_NSRLDb);
-        m_NSRLDb = NULL;
-    }
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbOpenNat(JNIEnv * env,
+    jclass obj, jstring pathJ) 
+{
     TSK_TCHAR pathT[1024];
     toTCHAR(env, pathT, 1024, pathJ);
-
-    TSK_HDB_OPEN_ENUM flags = TSK_HDB_OPEN_IDXONLY;
-    TSK_HDB_INFO * tempdb = tsk_hdb_open(pathT, flags);
-
-    if(tempdb == NULL)
+    TSK_HDB_INFO *db = tsk_hdb_open(pathT, TSK_HDB_OPEN_NONE);
+    if (!db)
     {
-        setThrowTskCoreError(env);
+        setThrowTskCoreError(env, tsk_error_get_errstr());
         return -1;
     }
     
-    m_NSRLDb = tempdb;
-    
+    // The index of the pointer in the vector is used as a handle for the
+    // database.
+    hashDbs.push_back(db);
+    return (jint)hashDbs.size();
+}
+
+/**
+ * Creates a new hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param pathJ The path to the hash database.
+ * @return A handle for the hash database.
+ */
+JNIEXPORT jint JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbNewNat(JNIEnv * env,
+    jclass obj, jstring pathJ)
+{
+    TSK_TCHAR pathT[1024];
+    toTCHAR(env, pathT, 1024, pathJ);
+    if (tsk_hdb_create(pathT)) {
+        setThrowTskCoreError(env, tsk_error_get_errstr());
+        return -1;
+    }
+
+    TSK_HDB_INFO *db = tsk_hdb_open(pathT, TSK_HDB_OPEN_NONE);
+    if (!db) {
+        setThrowTskCoreError(env, tsk_error_get_errstr());
+        return -1;
+    }
+
+    // The index of the pointer in the vector is used as a handle for the
+    // database.
+    hashDbs.push_back(db);    
+    return (jint)hashDbs.size();
+}
+
+/**
+ * Begins a hash database transaction.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return 1 on error and 0 on success.
+ */
+JNIEXPORT jint JNICALL
+Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbBeginTransactionNat(
+    JNIEnv *env, jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle - 1);
+    if (!db) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    return tsk_hdb_begin_transaction(db);
+}
+
+/**
+ * Commits a hash database transaction.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return 1 on error and 0 on success.
+ */
+JNIEXPORT jint JNICALL
+Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbCommitTransactionNat(
+    JNIEnv *env, jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle - 1);
+    if (!db) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    return tsk_hdb_commit_transaction(db);
+}
+
+/**
+ * Rolls back a hash database transaction.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return 1 on error and 0 on success.
+ */
+JNIEXPORT jint JNICALL
+Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbRollbackTransactionNat(
+    JNIEnv *env, jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (!db) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    return tsk_hdb_rollback_transaction(db);
+}
+
+/**
+ * Adds data to a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param filenameJ Name of the file that was hashed (can be null).
+ * @param hashMd5J MD5 hash of file contents (can be null).
+ * @param hashSha1J SHA-1 hash of file contents (can be null).
+ * @param hashSha256J Text of SHA256 hash (can be null).
+ * @param dbHandle A handle for the hash database.
+ * @return 1 on error and 0 on success.
+ */
+JNIEXPORT jint JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbAddEntryNat(JNIEnv * env,
+    jclass obj, jstring filenameJ, jstring hashMd5J, jstring hashSha1J, jstring hashSha256J,
+    jstring commentJ, jint dbHandle)
+{
+    if((size_t) dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    TSK_HDB_INFO * db = hashDbs.at(dbHandle-1);
+    if(!db) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return 1;
+    }
+
+    if(!db->accepts_updates()) {
+        setThrowTskCoreError(env, "Database does not accept updates");
+        return 1;
+    }
+
+    jboolean isCopy;
+    const char * name = filenameJ ? (const char *) env->GetStringUTFChars(filenameJ, &isCopy) : NULL;
+    const char * md5 = hashMd5J ? (const char *) env->GetStringUTFChars(hashMd5J, &isCopy) : NULL;
+    const char * sha1 = hashSha1J ? (const char *) env->GetStringUTFChars(hashSha1J, &isCopy) : NULL;
+    const char * sha256 = hashSha256J ? (const char *) env->GetStringUTFChars(hashSha256J, &isCopy) : NULL;
+    const char * comment = commentJ ? (const char *) env->GetStringUTFChars(commentJ, &isCopy) : NULL;
+   
+    if (tsk_hdb_add_entry(db, name, md5, sha1, sha256, comment)) {
+        setThrowTskCoreError(env, tsk_error_get_errstr());
+    }
+
+    if (filenameJ) {
+        env->ReleaseStringUTFChars(filenameJ, (const char *) name);
+    }
+
+    if (hashMd5J) { 
+        env->ReleaseStringUTFChars(hashMd5J, (const char *) md5);
+    }
+
+    if (hashSha1J) {
+        env->ReleaseStringUTFChars(hashSha1J, (const char *) sha1);
+    }
+
+    if (hashSha256J) {
+        env->ReleaseStringUTFChars(hashSha256J, (const char *) sha256);
+    }
+
+    if (commentJ) {
+        env->ReleaseStringUTFChars(commentJ, (const char *) comment);
+    }
+
     return 0;
 }
 
-/*
- * Set the "known bad" database to use for hash lookups.
- * @param env pointer to java environment this was called from
- * @param obj the java object this was called from
- * @param pathJ the path to the database
- * @return a handle for the known bad database
+/**
+ * Queries whether or not a hash database accepts updates.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return True if hash database can be updated.
  */
-JNIEXPORT jint JNICALL
-    Java_org_sleuthkit_datamodel_SleuthkitJNI_addDbKnownBadNat(JNIEnv * env,
-    jclass obj, jstring pathJ) {
-
-    TSK_TCHAR pathT[1024];
-    toTCHAR(env, pathT, 1024, pathJ);
-
-    TSK_HDB_OPEN_ENUM flags = TSK_HDB_OPEN_IDXONLY;
-    TSK_HDB_INFO * temp = tsk_hdb_open(pathT, flags);
-
-    if(temp == NULL)
-    {
-        setThrowTskCoreError(env);
-        return -1;
+JNIEXPORT jboolean JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbIsUpdateableNat(JNIEnv * env,
+    jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
     }
 
-    m_knownbads.push_back(temp);
-    
-    return m_knownbads.size();
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
+
+    return (jboolean)(tsk_hdb_accepts_updates(db) == static_cast<uint8_t>(1));
 }
 
-/*
- * Get the name of the database pointed to by path
- * @param env pointer to java environment this was called from
- * @param obj the java object this was called from
- * @param pathJ the path to the database
+/**
+ * Queries whether or not a hash database can be re-indexed. Only text-format
+ * databases with external indexes can be re-indexed.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return True if hash database can be indexed.
+ */
+JNIEXPORT jboolean JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbIsReindexableNat(JNIEnv * env,
+    jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
+
+    return (jboolean)((tsk_hdb_uses_external_indexes(db) == static_cast<uint8_t>(1)) && 
+                      (!tsk_hdb_is_idx_only(db) == static_cast<uint8_t>(1)));
+}
+ 
+/**
+ * Gets the path of a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return Path to the hash database or "None" if no path is available.
  */
 JNIEXPORT jstring JNICALL
-    Java_org_sleuthkit_datamodel_SleuthkitJNI_getDbName(JNIEnv * env,
-    jclass obj, jstring pathJ) {
-
-    TSK_HDB_OPEN_ENUM flags;
-    TSK_TCHAR pathT[1024];
-    toTCHAR(env, pathT, 1024, pathJ);
-    struct STAT_STR buffer;
-
-    if( TSTAT(pathT, &buffer) != -1 )
-        flags = TSK_HDB_OPEN_NONE;
-    else
-        flags = TSK_HDB_OPEN_IDXONLY;
-
-    TSK_HDB_INFO * tempdb = tsk_hdb_open(pathT, flags);
-
-    if(tempdb == NULL)
-    {
-        setThrowTskCoreError(env);
-        return env->NewStringUTF("-1");
-    }
-
-    jstring jname = env->NewStringUTF(tempdb->db_name);
-
-    tsk_hdb_close(tempdb);
-    return jname;
-}
-
-
-JNIEXPORT void JNICALL
-    Java_org_sleuthkit_datamodel_SleuthkitJNI_closeDbLookupsNat(JNIEnv * env,
-    jclass obj) {
-
-    if (m_NSRLDb != NULL) {
-        tsk_hdb_close(m_NSRLDb);
-        m_NSRLDb = NULL;
-    }
-
-    for_each(m_knownbads.begin(), m_knownbads.end(), tsk_hdb_close);
-   
-    m_knownbads.clear();
-}
-
-/*
- * Class:     org_sleuthkit_datamodel_SleuthkitJNI
- * Method:    nsrlDbLookup
- * Signature: (Ljava/lang/String;)I
- */
-JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_nsrlDbLookup
-(JNIEnv * env, jclass obj, jstring hash){
-
-    jboolean isCopy;
-
-    const char *md5 = (const char *) env->GetStringUTFChars(hash, &isCopy);
-
-    TSK_DB_FILES_KNOWN_ENUM file_known = TSK_DB_FILES_KNOWN_UNKNOWN;
-
-    if (m_NSRLDb != NULL) {
-        int8_t retval = tsk_hdb_lookup_str(m_NSRLDb, md5, TSK_HDB_FLAG_QUICK, NULL, NULL);
-
-        if (retval == -1) {
-            setThrowTskCoreError(env);
-        } else if (retval) {
-            file_known = TSK_DB_FILES_KNOWN_KNOWN;
-        }
-    }
-
-    env->ReleaseStringUTFChars(hash, (const char *) md5);
-
-    return (int) file_known;
-}
-
-/*
- * Class:     org_sleuthkit_datamodel_SleuthkitJNI
- * Method:    knownBadDbLookup
- * Signature: (Ljava/lang/String;)I
- */
-JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_knownBadDbLookup
-(JNIEnv * env, jclass obj, jstring hash, jint dbHandle){
-
-    if((size_t) dbHandle > m_knownbads.size()) {
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbPathNat(JNIEnv * env,
+    jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
         setThrowTskCoreError(env, "Invalid database handle");
-        return -1;
+        return NULL;
     }
 
-    jboolean isCopy;
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
 
-    const char *md5 = (const char *) env->GetStringUTFChars(hash, &isCopy);
+    jstring jPath = NULL;
+    const TSK_TCHAR *dbPath = tsk_hdb_get_db_path(db);
+    if (NULL != dbPath) {
+        const size_t pathLength = TSTRLEN(dbPath);
+        char *cPath = (char*)tsk_malloc((pathLength + 1) * sizeof(char));
+        snprintf(cPath, pathLength + 1, "%" PRIttocTSK, dbPath);
+        jPath = env->NewStringUTF(cPath);
+        free(cPath);
+    }
+    else {
+        jPath = env->NewStringUTF("None");
+    }
+    return jPath;
+}
 
-    TSK_DB_FILES_KNOWN_ENUM file_known = TSK_DB_FILES_KNOWN_UNKNOWN;
+/*
+ * Gets the path of the external MD5 hash index for a text-format database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return Path to the requested index or "None" if no path is available.
+ */
+JNIEXPORT jstring JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbIndexPathNat(JNIEnv * env,
+    jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
 
-    
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
 
-    TSK_HDB_INFO * db = m_knownbads.at(dbHandle-1);
+    // Currently only supporting md5 indexes through Java binding.
+    jstring jPath = NULL;
+    const TSK_TCHAR *indexPath = tsk_hdb_get_idx_path(db, TSK_HDB_HTYPE_MD5_ID);
+    if (NULL != indexPath) {
+        const size_t pathLength = TSTRLEN(indexPath);
+        char *cPath = (char*)tsk_malloc((pathLength + 1) * sizeof(char));
+        snprintf(cPath, pathLength + 1, "%" PRIttocTSK, indexPath);
+        jPath = env->NewStringUTF(cPath);
+        free(cPath);
+    }
+    else {
+        jPath = env->NewStringUTF("None");
+    }
+    return jPath;
+}
 
-    if(db != NULL) {
-        int8_t retval = tsk_hdb_lookup_str(db, md5, TSK_HDB_FLAG_QUICK, NULL, NULL);
+/**
+ * Queries whether the hash database is actually an external index for a
+ * text-format database that is being used for simple yes/no look ups in
+ * place of the roginal hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return True if the hash database is an external index serving as a 
+ * database.
+ */
+JNIEXPORT jboolean JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbIsIdxOnlyNat(JNIEnv * env,
+    jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
 
-        if (retval == -1) {
-            setThrowTskCoreError(env);
-        } else if (retval) {
-            file_known = TSK_DB_FILES_KNOWN_KNOWN_BAD;
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
+
+    return (jboolean)(tsk_hdb_is_idx_only(db) == static_cast<uint8_t>(1));
+}
+
+/**
+ * Gets the display name of a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return The display name.
+ */
+JNIEXPORT jstring JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbGetDisplayName
+  (JNIEnv * env, jclass obj, jint dbHandle)
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
+
+    jstring j_name = NULL;
+    const char *db_name = tsk_hdb_get_display_name(db);
+    if (NULL != db_name) {
+        j_name = env->NewStringUTF(db_name);
+    }
+    return j_name;
+}
+
+/**
+ * Closes all open hash databases.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ */
+JNIEXPORT void JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbCloseAll(JNIEnv * env,
+    jclass obj) 
+{
+    for (std::vector<TSK_HDB_INFO *>::iterator it = hashDbs.begin(); it != hashDbs.end(); ++it) {
+        if (NULL != *it) {
+            tsk_hdb_close(*it);
         }
     }
 
-    env->ReleaseStringUTFChars(hash, (const char *) md5);
+    hashDbs.clear();
+}
 
-    return (int) file_known;
+/**
+ * Closes a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ */
+JNIEXPORT void JNICALL
+    Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbClose(JNIEnv * env,
+    jclass obj, jint dbHandle) 
+{
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return;
+    }
+
+    tsk_hdb_close(db);
+
+    // Do NOT erase the element because that would shift the indices,
+    // messing up the existing handles.
+    hashDbs.at(dbHandle-1) = NULL;
+}
+
+/**
+ * Looks up a hash in a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return True if the hash is found in the hash database, false otherwise.
+ */
+JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookup
+(JNIEnv * env, jclass obj, jstring hash, jint dbHandle) 
+{
+    if ((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
+
+    jboolean isCopy;
+    const char *cHashStr = (const char *) env->GetStringUTFChars(hash, &isCopy);
+    jboolean file_known = false;
+    int8_t retval = tsk_hdb_lookup_str(db, cHashStr, TSK_HDB_FLAG_QUICK, NULL, NULL);
+    if (retval == -1) {
+        setThrowTskCoreError(env, tsk_error_get_errstr());
+    } 
+    else if (retval) {
+        file_known = true;
+    }
+
+    env->ReleaseStringUTFChars(hash, (const char *) cHashStr);
+
+    return file_known;
+}
+
+/**
+ * Looks up a hash in a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return A HashInfo object if the hash is found, NULL otherwise.
+ */
+JNIEXPORT jobject JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbLookupVerbose
+(JNIEnv * env, jclass obj, jstring hash, jint dbHandle) {
+    if ((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
+
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return NULL;
+    }
+    
+    jboolean isCopy;
+    const char *inputHash = (const char *) env->GetStringUTFChars(hash, &isCopy);
+    TskHashInfo result; 
+    int8_t returnCode = tsk_hdb_lookup_verbose_str(db, inputHash, (void*)&result);
+    env->ReleaseStringUTFChars(hash, (const char *) inputHash);
+    
+    if (returnCode == -1) {
+        setThrowTskCoreError(env, tsk_error_get_errstr());
+        return NULL;
+    }
+    else if (returnCode == 0) {
+        return NULL;
+    }
+
+    // Convert the hashes from the hash database so they can be written into
+    // the Java version of a HashInfo object.
+    const char *md5 = result.hashMd5.c_str();
+    jstring md5j = env->NewStringUTF(md5);
+            
+    const char *sha1 = result.hashSha1.c_str();
+    jstring sha1j = env->NewStringUTF(sha1);
+            
+    const char *sha256 = result.hashSha2_256.c_str();
+    jstring sha256j = env->NewStringUTF(sha256);
+
+    // Create and return a Java HashInfo object.
+    jclass clazz;
+    clazz = env->FindClass("org/sleuthkit/datamodel/HashHitInfo");
+    jmethodID ctor = env->GetMethodID(clazz, "<init>", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V");
+    jmethodID addName = env->GetMethodID(clazz, "addName", "(Ljava/lang/String;)V");
+    jmethodID addComment = env->GetMethodID(clazz, "addComment", "(Ljava/lang/String;)V");
+    jobject hashInfo = env->NewObject(clazz, ctor, md5j, sha1j, sha256j);
+    for (std::vector<std::string>::iterator it = result.fileNames.begin(); it != result.fileNames.end(); ++it) {
+        jstring namej = env->NewStringUTF((*it).c_str());
+        env->CallVoidMethod(hashInfo, addName, namej);
+    }
+    for (std::vector<std::string>::iterator it = result.comments.begin(); it != result.comments.end(); ++it) {
+        jstring commentj = env->NewStringUTF((*it).c_str());
+        env->CallVoidMethod(hashInfo, addComment, commentj);
+    }
+    return hashInfo;
 }
 
 /*
@@ -886,7 +1274,7 @@ copyBufToByteArray(JNIEnv * env, const char *buf, ssize_t len)
 inline static ssize_t
 copyBufToByteArray(JNIEnv * env, jbyteArray jbuf, const char *buf, ssize_t len)
 {
-    env->SetByteArrayRegion(jbuf, 0, len, (jbyte*)buf);
+    env->SetByteArrayRegion(jbuf, 0, (jsize)len, (jbyte*)buf);
     return len;
 }
 
@@ -950,7 +1338,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readImgNat(JNIEnv * env,
 	if (copiedbytes == -1) {
         setThrowTskCoreError(env, tsk_error_get());
     }
-    return copiedbytes;
+    return (jint)copiedbytes;
 }
 
 
@@ -1014,7 +1402,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readVsNat(JNIEnv * env,
     if (copiedbytes == -1) {
         setThrowTskCoreError(env, tsk_error_get());
     }
-    return copiedbytes;
+    return (jint)copiedbytes;
 }
 
 
@@ -1079,7 +1467,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readVolNat(JNIEnv * env,
     if (copiedbytes == -1) {
         setThrowTskCoreError(env, tsk_error_get());
     }
-    return copiedbytes;
+    return (jint)copiedbytes;
 }
 
 
@@ -1143,7 +1531,7 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readFsNat(JNIEnv * env,
     if (copiedbytes == -1) {
         setThrowTskCoreError(env, tsk_error_get());
     }
-    return copiedbytes;
+    return (jint)copiedbytes;
 }
 
 
@@ -1212,10 +1600,52 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_readFileNat(JNIEnv * env,
     if (copiedbytes == -1) {
         setThrowTskCoreError(env, tsk_error_get());
     }
-    return copiedbytes;
+    return (jint)copiedbytes;
 }
 
 
+/**
+ * Runs istat on a given file and saves the output to a temp file.
+ *
+ * @returns -1 on error (and throws exception)
+ */
+JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_saveFileMetaDataTextNat
+  (JNIEnv *env, jclass obj, jlong a_file_handle, jstring a_tmp_path)
+{
+    const TSK_JNI_FILEHANDLE *file_handle = castFsFile(env, a_file_handle);
+    if (file_handle == 0) {
+        //exception already set
+        return -1;
+    }
+    
+    // check the pointers
+    if (file_handle->fs_file == NULL || file_handle->fs_file->fs_info == NULL || file_handle->fs_file->meta == NULL) {
+        setThrowTskCoreError(env, "NULL pointers for istat file.");
+        return -1;
+    }
+    TSK_FS_INFO *fs_info = file_handle->fs_file->fs_info;   
+
+    // open a file to write the details to
+    jboolean isCopy;
+    char *str8 = (char *) env->GetStringUTFChars(a_tmp_path, &isCopy);
+    FILE *hFile = fopen(str8, "w");
+    if (hFile == NULL) {
+        env->ReleaseStringUTFChars(a_tmp_path, str8);
+        setThrowTskCoreError(env, "Couldn't open istat temp file for writing.");
+        return -1;
+    }
+    env->ReleaseStringUTFChars(a_tmp_path, str8);
+    
+    if (fs_info->istat(fs_info, hFile, file_handle->fs_file->meta->addr, 0, 0) != 0) {
+        fclose(hFile);
+        setThrowTskCoreError(env);
+        return -1;
+    }
+
+    fclose(hFile);
+    return 0;
+}
+
 /*
  * Close the given image
  * @param env pointer to java environment this was called from
@@ -1343,100 +1773,70 @@ Java_org_sleuthkit_datamodel_SleuthkitJNI_startVerboseLoggingNat
 }
 
 /*
- * Create an index for the given database path
- * @param env pointer to java environment this was called from
- * @param obj the java object this was called from
- * @param dbPathJ path for the database
+ * Creates an MD5 index for a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
  */
 JNIEXPORT void JNICALL
-Java_org_sleuthkit_datamodel_SleuthkitJNI_createLookupIndexNat (JNIEnv * env,
-    jclass obj, jstring dbPathJ)
+Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbCreateIndexNat (JNIEnv * env,
+    jclass obj, jint dbHandle)
 {
-    TSK_TCHAR dbPathT[1024];
-    toTCHAR(env, dbPathT, 1024, dbPathJ);
-
-    TSK_HDB_OPEN_ENUM flags = TSK_HDB_OPEN_NONE;
-    TSK_HDB_INFO * temp = tsk_hdb_open(dbPathT, flags);
-    if (temp == NULL) {
-        setThrowTskCoreError(env, "Error opening database to create index");
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
         return;
     }
 
-    TSK_TCHAR dbType[1024];
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return;
+    }
 
-    if(temp->db_type == TSK_HDB_DBTYPE_MD5SUM_ID) {
-        TSNPRINTF(dbType, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_MD5SUM_STR);
+    TSK_TCHAR idx_type[1024];
+    if(db->db_type == TSK_HDB_DBTYPE_MD5SUM_ID) {
+        TSNPRINTF(idx_type, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_MD5SUM_STR);
     }
-    else if(temp->db_type == TSK_HDB_DBTYPE_HK_ID) {
-        TSNPRINTF(dbType, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_HK_STR);
+    else if(db->db_type == TSK_HDB_DBTYPE_HK_ID) {
+        TSNPRINTF(idx_type, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_HK_STR);
     }
-    else if(temp->db_type == TSK_HDB_DBTYPE_ENCASE_ID) {
-        TSNPRINTF(dbType, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_ENCASE_STR);
+    else if(db->db_type == TSK_HDB_DBTYPE_ENCASE_ID) {
+        TSNPRINTF(idx_type, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_ENCASE_STR);
     }
     else {
-        TSNPRINTF(dbType, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_NSRL_MD5_STR);
+        // The Java bindings only support the generation of md5 indexes for
+        // an NSRL hash database.
+        TSNPRINTF(idx_type, 1024, _TSK_T("%") PRIcTSK, TSK_HDB_DBTYPE_NSRL_MD5_STR);
     }
-
-    if (tsk_hdb_makeindex(temp, dbType)) {
-        setThrowTskCoreError(env, "Error creating index");
+  
+    if (tsk_hdb_make_index(db, idx_type) != 0) {
+        setThrowTskCoreError(env, tsk_error_get_errstr());
     }
-
-    tsk_hdb_close(temp);
 }
 
 /*
- * Check if an index exists for the given database path.
- * @param env pointer to java environment this was called from
- * @param obj the java object this was called from
- * @param dbPathJ path for the database
+ * Queries whether or not an index for MD5 look ups exists for a hash database.
+ * @param env Pointer to Java environment from which this method was called.
+ * @param obj The Java object from which this method was called.
+ * @param dbHandle A handle for the hash database.
+ * @return True if the index exists.
  */
-JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_lookupIndexExistsNat
-  (JNIEnv * env, jclass obj, jstring dbPathJ) {
-
-    TSK_TCHAR dbPathT[1024];
-    toTCHAR(env, dbPathT, 1024, dbPathJ);
-
-    TSK_HDB_OPEN_ENUM flags = TSK_HDB_OPEN_IDXONLY;
-    TSK_HDB_INFO * temp = tsk_hdb_open(dbPathT, flags);
-    if (temp == NULL) {
-        return (jboolean) false;
+JNIEXPORT jboolean JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_hashDbIndexExistsNat
+  (JNIEnv * env, jclass obj, jint dbHandle) {
+    if((size_t)dbHandle > hashDbs.size()) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
     }
 
-    uint8_t retval = tsk_hdb_hasindex(temp, TSK_HDB_HTYPE_MD5_ID);
+    TSK_HDB_INFO *db = hashDbs.at(dbHandle-1);
+    if (db == NULL) {
+        setThrowTskCoreError(env, "Invalid database handle");
+        return (jboolean)false;
+    }
 
-    tsk_hdb_close(temp);
-    return (jboolean) retval == 1;
+    return (jboolean)(db->has_index(db, TSK_HDB_HTYPE_MD5_ID) == 1);
 }
 
-/*
- * Get the size of the index for the database at the given path
- * @param env pointer to java environment this was called from
- * @param obj the java object this was called from
- * @param dbPathJ the path for the database
- * @return -1 on error, otherwise size of index
- */
-JNIEXPORT jint JNICALL Java_org_sleuthkit_datamodel_SleuthkitJNI_getIndexSizeNat
-  (JNIEnv * env, jclass obj, jstring dbPathJ) {
-
-    TSK_TCHAR dbPathT[1024];
-    toTCHAR(env, dbPathT, 1024, dbPathJ);
-
-    TSK_HDB_OPEN_ENUM flags = TSK_HDB_OPEN_IDXONLY;
-    TSK_HDB_INFO * temp = tsk_hdb_open(dbPathT, flags);
-    if (temp == NULL) {
-        return -1;
-    }
-
-    if(tsk_hdb_hasindex(temp, TSK_HDB_HTYPE_MD5_ID)) {
-        return (jint) ((temp->idx_size - temp->idx_off) / (temp->idx_llen));
-    }
-
-
-    tsk_hdb_close(temp);
-    return -1;
-}
-
-
 /*
  * Query and get size of the device (such as physical disk, or image) pointed by the path
  * Might require elevated priviletes to work (otherwise will error)
