@@ -21,9 +21,12 @@ package org.sleuthkit.datamodel;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.RandomAccessFile;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.ResourceBundle;
 import java.util.Set;
+import java.util.SortedSet;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 import org.sleuthkit.datamodel.TskData.FileKnown;
@@ -46,6 +49,7 @@ public abstract class AbstractFile extends AbstractContent {
 	protected final Set<TSK_FS_META_FLAG_ENUM> metaFlags;
 	protected long size;
 	protected final long metaAddr, ctime, crtime, atime, mtime;
+	protected final int metaSeq;
 	protected final int uid, gid;
 	protected final short attrId;
 	protected final TskData.TSK_FS_ATTR_TYPE_ENUM attrType;
@@ -71,6 +75,7 @@ public abstract class AbstractFile extends AbstractContent {
 	 */
 	protected String md5Hash;
 	private static final Logger logger = Logger.getLogger(AbstractFile.class.getName());
+	private static final ResourceBundle bundle = ResourceBundle.getBundle("org.sleuthkit.datamodel.Bundle");
 
 	/**
 	 * Initializes common fields used by AbstactFile implementations (objects in
@@ -83,6 +88,7 @@ public abstract class AbstractFile extends AbstractContent {
 	 * @param name name field of the file
 	 * @param fileType type of the file
 	 * @param metaAddr
+	 * @param metaSeq
 	 * @param dirType
 	 * @param metaType
 	 * @param dirFlag
@@ -101,7 +107,7 @@ public abstract class AbstractFile extends AbstractContent {
 	 * @param parentPath
 	 */
 	protected AbstractFile(SleuthkitCase db, long objId, TskData.TSK_FS_ATTR_TYPE_ENUM attrType, short attrId,
-			String name, TskData.TSK_DB_FILES_TYPE_ENUM fileType, long metaAddr,
+			String name, TskData.TSK_DB_FILES_TYPE_ENUM fileType, long metaAddr, int metaSeq,
 			TSK_FS_NAME_TYPE_ENUM dirType, TSK_FS_META_TYPE_ENUM metaType, TSK_FS_NAME_FLAG_ENUM dirFlag, short metaFlags,
 			long size, long ctime, long crtime, long atime, long mtime, short modes, int uid, int gid, String md5Hash, FileKnown knownState,
 			String parentPath) {
@@ -110,6 +116,7 @@ public abstract class AbstractFile extends AbstractContent {
 		this.attrId = attrId;
 		this.fileType = fileType;
 		this.metaAddr = metaAddr;
+		this.metaSeq = metaSeq;
 		this.dirType = dirType;
 		this.metaType = metaType;
 		this.dirFlag = dirFlag;
@@ -125,7 +132,7 @@ public abstract class AbstractFile extends AbstractContent {
 
 		this.md5Hash = md5Hash;
 		if (knownState == null) {
-			this.knownState = FileKnown.UKNOWN;
+			this.knownState = FileKnown.UNKNOWN;
 		} else {
 			this.knownState = knownState;
 		}
@@ -258,6 +265,16 @@ public abstract class AbstractFile extends AbstractContent {
 		return metaAddr;
 	}
 
+	/**
+	 * Get the file meta address sequence. Only useful with NTFS. Incremented
+	 * each time a structure is re-allocated.
+	 *
+	 * @return Address of the meta data structure sequence.
+	 */
+	public long getMetaSeq() {
+		return metaSeq;
+	}
+
 	/**
 	 * Get the file's mode as a user-displayable string
 	 *
@@ -288,89 +305,89 @@ public abstract class AbstractFile extends AbstractContent {
 
 		// second and third characters = user permissions
 		if ((mode & irusr) == irusr) {
-			result += "r";
+			result += "r"; //NON-NLS
 		} else {
-			result += "-";
+			result += "-"; //NON-NLS
 		}
 		if ((mode & iwusr) == iwusr) {
-			result += "w";
+			result += "w"; //NON-NLS
 		} else {
-			result += "-";
+			result += "-"; //NON-NLS
 		}
 
 		// fourth character = set uid
 		if ((mode & isuid) == isuid) {
 			if ((mode & ixusr) == ixusr) {
-				result += "s";
+				result += "s"; //NON-NLS
 			} else {
-				result += "S";
+				result += "S"; //NON-NLS
 			}
 		} else {
 			if ((mode & ixusr) == ixusr) {
-				result += "x";
+				result += "x"; //NON-NLS
 			} else {
-				result += "-";
+				result += "-"; //NON-NLS
 			}
 		}
 
 		// fifth and sixth characters = group permissions
 		if ((mode & irgrp) == irgrp) {
-			result += "r";
+			result += "r"; //NON-NLS
 		} else {
-			result += "-";
+			result += "-"; //NON-NLS
 		}
 		if ((mode & iwgrp) == iwgrp) {
-			result += "w";
+			result += "w"; //NON-NLS
 		} else {
-			result += "-";
+			result += "-"; //NON-NLS
 		}
 
 		// seventh character = set gid
 		if ((mode & isgid) == isgid) {
 			if ((mode & ixgrp) == ixgrp) {
-				result += "s";
+				result += "s"; //NON-NLS
 			} else {
-				result += "S";
+				result += "S"; //NON-NLS
 			}
 		} else {
 			if ((mode & ixgrp) == ixgrp) {
-				result += "x";
+				result += "x"; //NON-NLS
 			} else {
-				result += "-";
+				result += "-"; //NON-NLS
 			}
 		}
 
 		// eighth and ninth character = other permissions
 		if ((mode & iroth) == iroth) {
-			result += "r";
+			result += "r"; //NON-NLS
 		} else {
-			result += "-";
+			result += "-"; //NON-NLS
 		}
 		if ((mode & iwoth) == iwoth) {
-			result += "w";
+			result += "w"; //NON-NLS
 		} else {
-			result += "-";
+			result += "-"; //NON-NLS
 		}
 
 		// tenth character = sticky bit
 		if ((mode & isvtx) == isvtx) {
 			if ((mode & ixoth) == ixoth) {
-				result += "t";
+				result += "t"; //NON-NLS
 			} else {
-				result += "T";
+				result += "T"; //NON-NLS
 			}
 		} else {
 			if ((mode & ixoth) == ixoth) {
-				result += "x";
+				result += "x"; //NON-NLS
 			} else {
-				result += "-";
+				result += "-"; //NON-NLS
 			}
 		}
 
 		// check the result
 		if (result.length() != 10) {
 			// throw error here
-			result = "ERROR";
+			result = "ERROR"; //NON-NLS
 		}
 		return result;
 	}
@@ -420,6 +437,33 @@ public abstract class AbstractFile extends AbstractContent {
 		return knownState;
 	}
 
+	/**
+	 * Figures out the extension from the filename, if there is one. We assume
+	 * that extensions only have ASCII alphanumeric chars
+	 *
+	 * @return filename extension in lowercase (not including the period) or
+	 * empty string if there is no extension
+	 */
+	public String getNameExtension() {
+		String ext;
+		int i = getName().lastIndexOf(".");
+		// > 0 because we assume it's not an extension if period is the first character
+		if ((i > 0) && ((i + 1) < getName().length())) {
+			ext = getName().substring(i + 1);
+		} else {
+			return "";
+		}
+
+		// we added this at one point to deal with files that had crazy names based on URLs
+		// it's too hard though to clean those up and not mess up basic extensions though.
+		// We need to add '-' to the below if we use it again
+//		String[] findNonAlphanumeric = ext.split("[^a-zA-Z0-9_]");
+//		if (findNonAlphanumeric.length > 1) {
+//			ext = findNonAlphanumeric[0];
+//		}		
+		return ext.toLowerCase();
+	}
+
 	/**
 	 * Get size of the file
 	 *
@@ -530,14 +574,14 @@ public abstract class AbstractFile extends AbstractContent {
 	public static String createNonUniquePath(String uniquePath) {
 
 		// split the path into parts
-		String[] pathSegments = uniquePath.split("/\\");
+		String[] pathSegments = uniquePath.split("/");
 
 		// see if uniquePath had an image and/or volume name
 		int index = 0;
-		if (pathSegments[0].startsWith("img_")) {
+		if (pathSegments[0].startsWith("img_")) { //NON-NLS
 			++index;
 		}
-		if (pathSegments[1].startsWith("vol_")) {
+		if (pathSegments[1].startsWith("vol_")) { //NON-NLS
 			++index;
 		}
 
@@ -554,6 +598,7 @@ public abstract class AbstractFile extends AbstractContent {
 	/**
 	 * @return a list of AbstractFiles that are the children of this Directory.
 	 * Only returns children of type TskData.TSK_DB_FILES_TYPE_ENUM.FS.
+	 * @throws org.sleuthkit.datamodel.TskCoreException
 	 */
 	public List<AbstractFile> listFiles() throws TskCoreException {
 		// first, get all children
@@ -632,37 +677,35 @@ public abstract class AbstractFile extends AbstractContent {
 	public boolean isMetaFlagSet(TSK_FS_META_FLAG_ENUM metaFlag) {
 		return metaFlags.contains(metaFlag);
 	}
-	
-	
+
 	@Override
 	public final int read(byte[] buf, long offset, long len) throws TskCoreException {
 		//template method
 		//if localPath is set, use local, otherwise, use readCustom() supplied by derived class
 		if (localPathSet) {
 			return readLocal(buf, offset, len);
-		}
-		else {
+		} else {
 			return readInt(buf, offset, len);
 		}
-		
+
 	}
-	
+
 	/**
-	 * Internal custom read  (non-local) method that child classes can implement
-	 * 
+	 * Internal custom read (non-local) method that child classes can implement
+	 *
 	 * @param buf buffer to read into
 	 * @param offset start reading position in the file
 	 * @param len number of bytes to read
 	 * @return number of bytes read
-	 * @throws TskCoreException exception thrown when file could not be read 
+	 * @throws TskCoreException exception thrown when file could not be read
 	 */
 	protected int readInt(byte[] buf, long offset, long len) throws TskCoreException {
 		return 0;
 	}
 
 	/**
-	 * Local file path read support 
-	 * 
+	 * Local file path read support
+	 *
 	 * @param buf buffer to read into
 	 * @param offset start reading position in the file
 	 * @param len number of bytes to read
@@ -671,19 +714,22 @@ public abstract class AbstractFile extends AbstractContent {
 	 */
 	protected final int readLocal(byte[] buf, long offset, long len) throws TskCoreException {
 		if (!localPathSet) {
-			throw new TskCoreException("Error reading local file, local path is not set");
+			throw new TskCoreException(
+					bundle.getString("AbstractFile.readLocal.exception.msg1.text"));
 		}
-		
+
 		if (isDir()) {
 			return 0;
 		}
 
-		getLocalFile();
+		loadLocalFile();
 		if (!localFile.exists()) {
-			throw new TskCoreException("Error reading local file, it does not exist at local path: " + localAbsPath);
+			throw new TskCoreException(
+					MessageFormat.format(bundle.getString("AbstractFile.readLocal.exception.msg2.text"), localAbsPath));
 		}
 		if (!localFile.canRead()) {
-			throw new TskCoreException("Error reading local file, file not readable at local path: " + localAbsPath);
+			throw new TskCoreException(
+					MessageFormat.format(bundle.getString("AbstractFile.readLocal.exception.msg3.text"), localAbsPath));
 		}
 
 		int bytesRead = 0;
@@ -694,7 +740,9 @@ public abstract class AbstractFile extends AbstractContent {
 					try {
 						localFileHandle = new RandomAccessFile(localFile, "r");
 					} catch (FileNotFoundException ex) {
-						final String msg = "Error reading local file: " + localAbsPath;
+						final String msg = MessageFormat.format(bundle.getString(
+								"AbstractFile.readLocal.exception.msg4.text"),
+								localAbsPath);
 						logger.log(Level.SEVERE, msg, ex);
 						//file could have been deleted or moved
 						throw new TskCoreException(msg, ex);
@@ -712,7 +760,7 @@ public abstract class AbstractFile extends AbstractContent {
 			//note, we are always writing at 0 offset of user buffer
 			bytesRead = localFileHandle.read(buf, 0, (int) len);
 		} catch (IOException ex) {
-			final String msg = "Cannot read local file: " + localAbsPath;
+			final String msg = MessageFormat.format(bundle.getString("AbstractFile.readLocal.exception.msg5.text"), localAbsPath);
 			logger.log(Level.SEVERE, msg, ex);
 			//local file could have been deleted / moved
 			throw new TskCoreException(msg, ex);
@@ -723,14 +771,15 @@ public abstract class AbstractFile extends AbstractContent {
 
 	/**
 	 * Set local path for the file, as stored in db tsk_files_path, relative to
-	 * the case db path or an absolute path.
-	 * When set, subsequent invocations of read() will read the file in the local path.
+	 * the case db path or an absolute path. When set, subsequent invocations of
+	 * read() will read the file in the local path.
 	 *
 	 * @param localPath local path to be set
-	 * @param isAbsolute true if the path is absolute, false if relative to the case db
+	 * @param isAbsolute true if the path is absolute, false if relative to the
+	 * case db
 	 */
 	protected void setLocalPath(String localPath, boolean isAbsolute) {
-		
+
 		if (localPath == null || localPath.equals("")) {
 			this.localPath = "";
 			localAbsPath = null;
@@ -739,17 +788,15 @@ public abstract class AbstractFile extends AbstractContent {
 			this.localPath = localPath;
 			if (isAbsolute) {
 				this.localAbsPath = localPath;
-			}
-			else {
+			} else {
 				this.localAbsPath = getSleuthkitCase().getDbDirPath() + java.io.File.separator + this.localPath;
 			}
 			this.localPathSet = true;
 		}
 	}
 
-
 	/**
-	 * Get local relative to case db path of the file 
+	 * Get local relative to case db path of the file
 	 *
 	 * @return local file path if set
 	 */
@@ -767,8 +814,8 @@ public abstract class AbstractFile extends AbstractContent {
 	}
 
 	/**
-	 * Check if the file exists. 
-	 * If non-local always true, if local, checks if actual local path exists
+	 * Check if the file exists. If non-local always true, if local, checks if
+	 * actual local path exists
 	 *
 	 * @return true if the file exists, false otherwise
 	 */
@@ -776,15 +823,20 @@ public abstract class AbstractFile extends AbstractContent {
 		if (!localPathSet) {
 			return true;
 		} else {
-			getLocalFile();
-			return localFile.exists();
+			try {
+				loadLocalFile();
+				return localFile.exists();
+			} catch (TskCoreException ex) {
+				logger.log(Level.SEVERE, ex.getMessage());
+				return false;
+			}
 		}
 	}
 
 	/**
-	 * Check if the file exists and is readable. 
-	 * If non-local (e.g. within an image), always true, if local,
-	 * checks if actual local path exists and is readable
+	 * Check if the file exists and is readable. If non-local (e.g. within an
+	 * image), always true, if local, checks if actual local path exists and is
+	 * readable
 	 *
 	 * @return true if the file is readable
 	 */
@@ -792,30 +844,36 @@ public abstract class AbstractFile extends AbstractContent {
 		if (!localPathSet) {
 			return true;
 		} else {
-			getLocalFile();
-			return localFile.canRead();
+			try {
+				loadLocalFile();
+				return localFile.canRead();
+			} catch (TskCoreException ex) {
+				logger.log(Level.SEVERE, ex.getMessage());
+				return false;
+			}
 		}
-
 	}
 
 	/**
-	 * Lazy load local file handle and return it, if localPath has been set
+	 * Lazy load local file handle
 	 *
-	 * @return java.io.File object representing the local file, or null if local path has not been set
 	 */
-	private java.io.File getLocalFile() {
+	private void loadLocalFile() throws TskCoreException {
 		if (!localPathSet) {
-			return null;
+			throw new TskCoreException(
+					bundle.getString("AbstractFile.readLocal.exception.msg1.text"));
 		}
 
-		if (localFile == null) {
-			synchronized (this) {
-				if (localFile == null) {
-					localFile = new java.io.File(localAbsPath);
-				}
+		// already been set
+		if (localFile != null) {
+			return;
+		}
+
+		synchronized (this) {
+			if (localFile == null) {
+				localFile = new java.io.File(localAbsPath);
 			}
 		}
-		return localFile;
 	}
 
 	@Override
@@ -828,7 +886,7 @@ public abstract class AbstractFile extends AbstractContent {
 					try {
 						localFileHandle.close();
 					} catch (IOException ex) {
-						logger.log(Level.SEVERE, "Could not close file handle for file: " + getParentPath() + "/" + getName(), ex);
+						logger.log(Level.SEVERE, "Could not close file handle for file: " + getParentPath() + "/" + getName(), ex); //NON-NLS
 					}
 					localFileHandle = null;
 				}
@@ -842,28 +900,28 @@ public abstract class AbstractFile extends AbstractContent {
 		try {
 			close();
 		} finally {
-			super.finalize(); 
+			super.finalize();
 		}
 	}
 
 	@Override
 	public String toString(boolean preserveState) {
-		return super.toString(preserveState) + "AbstractFile [\t"
-				+ "\t" + "fileType " + fileType
-				+ "\tctime " + ctime
-				+ "\tcrtime " + crtime
-				+ "\t" + "mtime " + mtime + "\t" + "atime " + atime
-				+ "\t" + "attrId " + attrId
-				+ "\t" + "attrType " + attrType
-				+ "\t" + "dirFlag " + dirFlag + "\t" + "dirType " + dirType
-				+ "\t" + "uid " + uid
-				+ "\t" + "gid " + gid
-				+ "\t" + "metaAddr " + metaAddr + "\t" + "metaFlags " + metaFlags
-				+ "\t" + "metaType " + metaType + "\t" + "modes " + modes
-				+ "\t" + "parentPath " + parentPath + "\t" + "size " + size
-				+ "\t" + "knownState " + knownState + "\t" + "md5Hash " + md5Hash
-				+ "\t" + "localPathSet " + localPathSet + "\t" + "localPath " + localPath
-				+ "\t" + "localAbsPath " + localAbsPath + "\t" + "localFile " + localFile
+		return super.toString(preserveState) + "AbstractFile [\t" //NON-NLS
+				+ "\t" + "fileType " + fileType //NON-NLS
+				+ "\tctime " + ctime //NON-NLS
+				+ "\tcrtime " + crtime //NON-NLS
+				+ "\t" + "mtime " + mtime + "\t" + "atime " + atime //NON-NLS
+				+ "\t" + "attrId " + attrId //NON-NLS
+				+ "\t" + "attrType " + attrType //NON-NLS
+				+ "\t" + "dirFlag " + dirFlag + "\t" + "dirType " + dirType //NON-NLS
+				+ "\t" + "uid " + uid //NON-NLS
+				+ "\t" + "gid " + gid //NON-NLS
+				+ "\t" + "metaAddr " + metaAddr + "\t" + "metaSeq " + metaSeq + "\t" + "metaFlags " + metaFlags //NON-NLS
+				+ "\t" + "metaType " + metaType + "\t" + "modes " + modes //NON-NLS
+				+ "\t" + "parentPath " + parentPath + "\t" + "size " + size //NON-NLS
+				+ "\t" + "knownState " + knownState + "\t" + "md5Hash " + md5Hash //NON-NLS
+				+ "\t" + "localPathSet " + localPathSet + "\t" + "localPath " + localPath //NON-NLS
+				+ "\t" + "localAbsPath " + localAbsPath + "\t" + "localFile " + localFile //NON-NLS
 				+ "]\t";
 	}
 
@@ -901,4 +959,39 @@ public abstract class AbstractFile extends AbstractContent {
 
 		return epoch;
 	}
+
+	/**
+	 * Possible return values for comparing a file to a list of mime types
+	 */
+	public enum MimeMatchEnum {
+
+		UNDEFINED, /// file does not have a defined mime time in blackboard
+		TRUE, /// file has a defined mime type and it is one of the given ones
+		FALSE /// file has a defined mime type and it is not one of the given ones.
+	}
+
+	/**
+	 * Determines if this file's type is one of the ones passed in. Uses the
+	 * blackboard attribute for file type.
+	 *
+	 * @param mimeTypes Set of file types to compare against
+	 * @return
+	 */
+	public MimeMatchEnum isMimeType(SortedSet<String> mimeTypes) {
+		try {
+			List<BlackboardAttribute> attrs = getGenInfoAttributes(BlackboardAttribute.ATTRIBUTE_TYPE.TSK_FILE_TYPE_SIG);
+			if (attrs.isEmpty()) {
+				return MimeMatchEnum.UNDEFINED;
+			}
+
+			for (BlackboardAttribute batt : attrs) {
+				if (mimeTypes.contains(batt.getValueString())) {
+					return MimeMatchEnum.TRUE;
+				}
+			}
+			return MimeMatchEnum.FALSE;
+		} catch (TskCoreException ex) {
+			return MimeMatchEnum.UNDEFINED;
+		}
+	}
 }
