@@ -95,6 +95,22 @@ public interface ContentVisitor<T> {
 	 */
 	T visit(VirtualDirectory ld);
 
+	/**
+	 * Act on (visit) a DerivedFile content object
+	 *
+	 * @param lf local file to visit / act on
+	 * @return result of the visit
+	 */
+	T visit(DerivedFile lf);
+
+	/**
+	 * Act on (visit) a LocalFile content object
+	 *
+	 * @param df derived file to visit / act on
+	 * @return result of the visit
+	 */
+	T visit(LocalFile df);
+
 	/**
 	 * The default content visitor - quickest method for implementing a custom
 	 * visitor. Every visit method delegates to the defaultVisit method, the
@@ -148,5 +164,15 @@ public interface ContentVisitor<T> {
 		public T visit(VirtualDirectory ld) {
 			return defaultVisit(ld);
 		}
+
+		@Override
+		public T visit(DerivedFile df) {
+			return defaultVisit(df);
+		}
+
+		@Override
+		public T visit(LocalFile lf) {
+			return defaultVisit(lf);
+		}
 	}
 }
