@@ -18,10 +18,15 @@
  */
 package org.sleuthkit.datamodel;
 
+import java.io.BufferedReader;
+import java.io.FileReader;
+import java.io.IOException;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
+import java.util.ArrayList;
 import java.util.GregorianCalendar;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 import java.util.TimeZone;
 import org.sleuthkit.datamodel.TskData.TSK_FS_ATTR_TYPE_ENUM;
@@ -50,19 +55,42 @@ public class SleuthkitJNI {
 
 	private static native void closeCaseDbNat(long db) throws TskCoreException;
 
-	private static native int setDbNSRLNat(String hashDbPath) throws TskCoreException;
+	private static native int hashDbOpenNat(String hashDbPath) throws TskCoreException;
 
-	private static native int addDbKnownBadNat(String hashDbPath) throws TskCoreException;
+	private static native int hashDbNewNat(String hashDbPath) throws TskCoreException;
 
-	private static native String getDbName(String hashDbPath) throws TskCoreException;
+	private static native int hashDbBeginTransactionNat(int dbHandle) throws TskCoreException;
 
-	private static native void closeDbLookupsNat() throws TskCoreException;
+	private static native int hashDbCommitTransactionNat(int dbHandle) throws TskCoreException;
 
-	private static native int knownBadDbLookup(String hash, int dbHandle) throws TskCoreException;
+	private static native int hashDbRollbackTransactionNat(int dbHandle) throws TskCoreException;
 
-	private static native int nsrlDbLookup(String hash) throws TskCoreException;
+	private static native int hashDbAddEntryNat(String filename, String hashMd5, String hashSha1, String hashSha256, String comment, int dbHandle) throws TskCoreException;
 
-	private static native int getIndexSizeNat(String hashDbPath) throws TskCoreException;
+	private static native boolean hashDbIsUpdateableNat(int dbHandle);
+
+	private static native boolean hashDbIsReindexableNat(int dbHandle);
+
+	private static native String hashDbPathNat(int dbHandle);
+
+	private static native String hashDbIndexPathNat(int dbHandle);
+
+	private static native String hashDbGetDisplayName(int dbHandle) throws TskCoreException;
+
+	private static native void hashDbCloseAll() throws TskCoreException;
+
+	private static native void hashDbClose(int dbHandle) throws TskCoreException;
+
+	//hash-lookup database functions   
+	private static native void hashDbCreateIndexNat(int dbHandle) throws TskCoreException;
+
+	private static native boolean hashDbIndexExistsNat(int dbHandle) throws TskCoreException;
+
+	private static native boolean hashDbIsIdxOnlyNat(int dbHandle) throws TskCoreException;
+
+	private static native boolean hashDbLookup(String hash, int dbHandle) throws TskCoreException;
+
+	private static native HashHitInfo hashDbLookupVerbose(String hash, int dbHandle) throws TskCoreException;
 
 	//load image
 	private static native long initAddImgNat(long db, String timezone, boolean processUnallocSpace, boolean noFatFsOrphans) throws TskCoreException;
@@ -97,6 +125,8 @@ public class SleuthkitJNI {
 
 	private static native int readFileNat(long fileHandle, byte[] readBuffer, long offset, long len) throws TskCoreException;
 
+	private static native int saveFileMetaDataTextNat(long fileHandle, String fileName) throws TskCoreException;
+
 	//close functions
 	private static native void closeImgNat(long imgHandle);
 
@@ -106,11 +136,6 @@ public class SleuthkitJNI {
 
 	private static native void closeFileNat(long fileHandle);
 
-	//hash-lookup database functions
-	private static native void createLookupIndexNat(String dbPath) throws TskCoreException;
-
-	private static native boolean lookupIndexExistsNat(String dbPath) throws TskCoreException;
-
 	//util functions
 	private static native long findDeviceSizeNat(String devicePath) throws TskCoreException;
 
@@ -119,9 +144,10 @@ public class SleuthkitJNI {
 	//Linked library loading
 	static {
 		LibraryUtils.loadSleuthkitJNI();
-    }
+	}
+
+	private SleuthkitJNI() {
 
-	public SleuthkitJNI() {
 	}
 
 	/**
@@ -150,35 +176,8 @@ public class SleuthkitJNI {
 		}
 
 		/**
-		 * Clear currently set lookup databases within TSK
-		 *
-		 * @throws TskCoreException exception thrown if critical error occurs
-		 * within TSK
+		 * ****************** Hash Database Methods **********************
 		 */
-		void clearLookupDatabases() throws TskCoreException {
-			closeDbLookupsNat();
-		}
-
-		/**
-		 * Set the NSRL database
-		 *
-		 * @param path The path to the database
-		 * @return a handle for that database
-		 */
-		int setNSRLDatabase(String path) throws TskCoreException {
-			return setDbNSRLNat(path);
-		}
-
-		/**
-		 * Add the known bad database
-		 *
-		 * @param path The path to the database
-		 * @return a handle for that database
-		 */
-		int addKnownBadDatabase(String path) throws TskCoreException {
-			return addDbKnownBadNat(path);
-		}
-
 		/**
 		 * Start the process of adding a disk image to the case
 		 *
@@ -301,7 +300,7 @@ public class SleuthkitJNI {
 			 * @return the currently processing directory
 			 */
 			public synchronized String currentDirectory() {
-				return autoDbPointer == 0 ? "NO_INFO" : getCurDirNat(autoDbPointer);
+				return autoDbPointer == 0 ? "NO_INFO" : getCurDirNat(autoDbPointer); //NON-NLS
 			}
 		}
 	}
@@ -530,6 +529,40 @@ public class SleuthkitJNI {
 		return readFileNat(fileHandle, readBuffer, offset, len);
 	}
 
+	/**
+	 * Get human readable (some what) details about a file. This is the same as
+	 * the 'istat' TSK tool
+	 *
+	 * @param fileHandle pointer to file structure in the sleuthkit
+	 * @return text
+	 * @throws TskCoreException if errors occurred
+	 */
+	public static List<String> getFileMetaDataText(long fileHandle) throws TskCoreException {
+		try {
+			java.io.File tmp = java.io.File.createTempFile("tsk", ".txt");
+
+			saveFileMetaDataTextNat(fileHandle, tmp.getAbsolutePath());
+
+			FileReader fr = new FileReader(tmp.getAbsolutePath());
+			BufferedReader textReader = new BufferedReader(fr);
+
+			List<String> lines = new ArrayList<String>();
+			while (true) {
+				String line = textReader.readLine();
+				if (line == null) {
+					break;
+				}
+				lines.add(line);
+			}
+			textReader.close();
+			fr.close();
+			tmp.delete();
+			return lines;
+		} catch (IOException ex) {
+			throw new TskCoreException("Error reading istat output: " + ex.getLocalizedMessage());
+		}
+	}
+
 	//free pointers
 	/**
 	 * frees the imgHandle pointer currently does not close the image, until the
@@ -573,14 +606,17 @@ public class SleuthkitJNI {
 		closeFileNat(fileHandle);
 	}
 
+	/**
+	 * **************************** Hash database methods ****************
+	 */
 	/**
 	 * Create an index for the given database path.
 	 *
 	 * @param dbPath The path to the database
 	 * @throws TskCoreException if a critical error occurs within TSK core
 	 */
-	public static void createLookupIndex(String dbPath) throws TskCoreException {
-		createLookupIndexNat(dbPath);
+	public static void createLookupIndexForHashDatabase(int dbHandle) throws TskCoreException {
+		hashDbCreateIndexNat(dbHandle);
 	}
 
 	/**
@@ -590,89 +626,168 @@ public class SleuthkitJNI {
 	 * @return true if index exists
 	 * @throws TskCoreException if a critical error occurs within TSK core
 	 */
-	public static boolean lookupIndexExists(String dbPath) throws TskCoreException {
-		return lookupIndexExistsNat(dbPath);
+	public static boolean hashDatabaseHasLookupIndex(int dbHandle) throws TskCoreException {
+		return hashDbIndexExistsNat(dbHandle);
 	}
 
 	/**
-	 * Set the NSRL database
+	 * hashDatabaseCanBeReindexed
+	 *
+	 * @param dbHandle previously opened hash db handle
+	 * @return Does this database have a source database that is different than
+	 * the index?
+	 * @throws TskCoreException if a critical error occurs within TSK core
+	 */
+	public static boolean hashDatabaseCanBeReindexed(int dbHandle) throws TskCoreException {
+		return hashDbIsReindexableNat(dbHandle);
+	}
+
+	/**
+	 * getHashDatabasePath
+	 *
+	 * @param dbHandle previously opened hash db handle
+	 * @return Hash db file path
+	 * @throws TskCoreException if a critical error occurs within TSK core
+	 */
+	public static String getHashDatabasePath(int dbHandle) throws TskCoreException {
+		return hashDbPathNat(dbHandle);
+	}
+
+	/**
+	 * getHashDatabaseIndexPath
+	 *
+	 * @param dbHandle previously opened hash db handle
+	 * @return Index file path
+	 * @throws TskCoreException if a critical error occurs within TSK core
+	 */
+	public static String getHashDatabaseIndexPath(int dbHandle) throws TskCoreException {
+		return hashDbIndexPathNat(dbHandle);
+	}
+
+	public static int openHashDatabase(String path) throws TskCoreException {
+		return hashDbOpenNat(path);
+	}
+
+	/**
+	 * Creates a hash database. Will be of the default TSK hash database type.
 	 *
 	 * @param path The path to the database
 	 * @return a handle for that database
 	 * @throws TskCoreException if a critical error occurs within TSK core
 	 */
-	public static int setNSRLDatabase(String path) throws TskCoreException {
-		return setDbNSRLNat(path);
+	public static int createHashDatabase(String path) throws TskCoreException {
+		return hashDbNewNat(path);
 	}
 
 	/**
-	 * Add the known bad database
+	 * Close the currently open lookup databases. Resets the handle counting.
 	 *
-	 * @param path The path to the database
-	 * @return a handle for that database
-	 * @throws TskCoreException if a critical error occurs within TSK core
+	 * @throws TskCoreException exception thrown if critical error occurs within
+	 * TSK
 	 */
-	public static int addKnownBadDatabase(String path) throws TskCoreException {
-		return addDbKnownBadNat(path);
+	public static void closeAllHashDatabases() throws TskCoreException {
+		hashDbCloseAll();
+	}
+
+	/**
+	 * Close a particular open lookup database. Existing handles are not
+	 * affected.
+	 *
+	 * @throws TskCoreException exception thrown if critical error occurs within
+	 * TSK
+	 */
+	public static void closeHashDatabase(int dbHandle) throws TskCoreException {
+		hashDbClose(dbHandle);
 	}
 
 	/**
 	 * Get the name of the database
 	 *
-	 * @param path The path to the database
-	 * @throws TskCoreException if a critical error occurs within TSK core
-	 */
-	public static String getDatabaseName(String path) throws TskCoreException {
-		return getDbName(path);
-	}
-
-	/**
-	 * Look up the given hash in the NSRL database
-	 *
-	 * @param hash
-	 * @return the status of the hash in the NSRL
-	 * @throws TskCoreException if a critical error occurs within TSK core
-	 */
-	public static TskData.FileKnown nsrlHashLookup(String hash) throws TskCoreException {
-		return TskData.FileKnown.valueOf((byte) nsrlDbLookup(hash));
-	}
-
-	/**
-	 * Look up the given hash in the known bad database
-	 *
-	 * @param hash
 	 * @param dbHandle previously opened hash db handle
-	 * @return the status of the hash in the known bad database
 	 * @throws TskCoreException if a critical error occurs within TSK core
 	 */
-	public static TskData.FileKnown knownBadHashLookup(String hash, int dbHandle) throws TskCoreException {
-		return TskData.FileKnown.valueOf((byte) knownBadDbLookup(hash, dbHandle));
+	public static String getHashDatabaseDisplayName(int dbHandle) throws TskCoreException {
+		return hashDbGetDisplayName(dbHandle);
 	}
 
 	/**
-	 * Get the size of the index of the given database
+	 * Lookup the given hash value and get basic answer
 	 *
-	 * @param path the path to the database
-	 * @return the size of the index or -1 if it doesn't exist
+	 * @param hash Hash value to search for
+	 * @param dbHandle Handle of database to lookup in.
+	 * @return True if hash was found in database.
 	 * @throws TskCoreException
 	 */
-	public static int getIndexSize(String path) throws TskCoreException {
-		return getIndexSizeNat(path);
+	public static boolean lookupInHashDatabase(String hash, int dbHandle) throws TskCoreException {
+		return hashDbLookup(hash, dbHandle);
 	}
 
 	/**
-	 * Convert this timezone from long to short form
-	 * Convert timezoneLongForm passed in from long to short form
+	 * Lookup hash value in DB and return details on results (more time
+	 * consuming than basic lookup)
+	 *
+	 * @param hash Hash value to search for
+	 * @param dbHandle Handle of database to lookup in.
+	 * @return Details on hash if it was in DB or null if it was not found.
+	 * @throws TskCoreException
+	 */
+	public static HashHitInfo lookupInHashDatabaseVerbose(String hash, int dbHandle) throws TskCoreException {
+		return hashDbLookupVerbose(hash, dbHandle);
+	}
+
+	/**
+	 * Adds a hash value to a hash database.
+	 *
+	 * @param filename Name of file (can be null)
+	 * @param md5 Text of MD5 hash (can be null)
+	 * @param sha1 Text of SHA1 hash (can be null)
+	 * @param sha256 Text of SHA256 hash (can be null)
+	 * @param dbHandle Handle to DB
+	 * @throws TskCoreException
+	 */
+	public static void addToHashDatabase(String filename, String md5, String sha1, String sha256, String comment, int dbHandle) throws TskCoreException {
+		hashDbAddEntryNat(filename, md5, sha1, sha256, comment, dbHandle);
+	}
+
+	public static void addToHashDatabase(List<HashEntry> hashes, int dbHandle) throws TskCoreException {
+		hashDbBeginTransactionNat(dbHandle);
+		try {
+			for (HashEntry entry : hashes) {
+				hashDbAddEntryNat(entry.getFileName(), entry.getMd5Hash(), entry.getSha1Hash(), entry.getSha256Hash(), entry.getComment(), dbHandle);
+			}
+			hashDbCommitTransactionNat(dbHandle);
+		} catch (TskCoreException ex) {
+			try {
+				hashDbRollbackTransactionNat(dbHandle);
+			} catch (TskCoreException ex2) {
+				ex2.initCause(ex);
+				throw ex2;
+			}
+			throw ex;
+		}
+	}
+
+	public static boolean isUpdateableHashDatabase(int dbHandle) throws TskCoreException {
+		return hashDbIsUpdateableNat(dbHandle);
+	}
+
+	public static boolean hashDatabaseIsIndexOnly(int dbHandle) throws TskCoreException {
+		return hashDbIsIdxOnlyNat(dbHandle);
+	}
+
+	/**
+	 * Convert this timezone from long to short form Convert timezoneLongForm
+	 * passed in from long to short form
 	 *
 	 * @param timezoneLongForm the long form (e.g., America/New_York)
-	 * @return the short form (e.g., EST5EDT) string representation, or an empty string if
-	 * empty long form was passed in
+	 * @return the short form (e.g., EST5EDT) string representation, or an empty
+	 * string if empty long form was passed in
 	 */
 	private static String timezoneLongToShort(String timezoneLongForm) {
-		if (timezoneLongForm  == null || timezoneLongForm.isEmpty()) {
+		if (timezoneLongForm == null || timezoneLongForm.isEmpty()) {
 			return "";
 		}
-		
+
 		String timezoneShortForm = "";
 		TimeZone zone = TimeZone.getTimeZone(timezoneLongForm);
 		int offset = zone.getRawOffset() / 1000;
