@@ -38,25 +38,25 @@ import org.sleuthkit.datamodel.TskData.TSK_FS_NAME_TYPE_ENUM;
 public class VirtualDirectory extends AbstractFile {
 
 	//some built-in virtual directory names
-	public static final String NAME_UNALLOC = "$Unalloc";
-	public static final String NAME_CARVED = "$CarvedFiles";
-	
-	protected VirtualDirectory(SleuthkitCase db, long objId, String name, TSK_FS_NAME_TYPE_ENUM dirType, 
-			TSK_FS_META_TYPE_ENUM metaType, TSK_FS_NAME_FLAG_ENUM dirFlag, short metaFlags, 
+	public static final String NAME_UNALLOC = "$Unalloc"; //NON-NLS
+	public static final String NAME_CARVED = "$CarvedFiles"; //NON-NLS
+
+	protected VirtualDirectory(SleuthkitCase db, long objId, String name, TSK_FS_NAME_TYPE_ENUM dirType,
+			TSK_FS_META_TYPE_ENUM metaType, TSK_FS_NAME_FLAG_ENUM dirFlag, short metaFlags,
 			long size, String md5Hash, FileKnown knownState, String parentPath) {
-		super(db, objId, TSK_FS_ATTR_TYPE_ENUM.TSK_FS_ATTR_TYPE_DEFAULT, (short)0, name, 
-				TskData.TSK_DB_FILES_TYPE_ENUM.VIRTUAL_DIR, 0L, dirType, metaType, dirFlag, 
-				metaFlags, 0L, 0L, 0L, 0L, 0L, (short)0, 0, 0, md5Hash, knownState, parentPath);
+		super(db, objId, TSK_FS_ATTR_TYPE_ENUM.TSK_FS_ATTR_TYPE_DEFAULT, (short) 0, name,
+				TskData.TSK_DB_FILES_TYPE_ENUM.VIRTUAL_DIR, 0L, 0, dirType, metaType, dirFlag,
+				metaFlags, 0L, 0L, 0L, 0L, 0L, (short) 0, 0, 0, md5Hash, knownState, parentPath);
 	}
 
 	@Override
 	public List<Content> getChildren() throws TskCoreException {
-		return getSleuthkitCase().getVirtualDirectoryChildren(this);
+		return getSleuthkitCase().getAbstractFileChildren(this);
 	}
 
 	@Override
 	public List<Long> getChildrenIds() throws TskCoreException {
-		return getSleuthkitCase().getLayoutDirectoryChildrenIds(this);
+		return getSleuthkitCase().getAbstractFileChildrenIds(this);
 	}
 
 	@Override
@@ -69,8 +69,6 @@ public class VirtualDirectory extends AbstractFile {
 		//nothing to be closed
 	}
 
-	
-
 	@Override
 	public boolean isRoot() {
 		return false;
@@ -87,20 +85,18 @@ public class VirtualDirectory extends AbstractFile {
 	}
 
 	@Override
-	public Image getImage() throws TskCoreException {
-		Content parent =  getParent();
+	public Content getDataSource() throws TskCoreException {
+		Content parent = getParent();
 		if (parent != null) {
-			return parent.getImage();
-		}
-		else {
+			return parent.getDataSource();
+		} else {
 			//root-level VirtualDirectory, such as local files container
-			return null;
+			return this;
 		}
-
 	}
 
 	@Override
-	public String toString(boolean preserveState){
-		return super.toString(preserveState) + "VirtualDirectory [\t" + "]\t";
-	}		
+	public String toString(boolean preserveState) {
+		return super.toString(preserveState) + "VirtualDirectory [\t" + "]\t"; //NON-NLS
+	}
 }
