@@ -51,7 +51,7 @@ public class VolumeSystem extends AbstractContent {
 	public int read(byte[] readBuffer, long offset, long len) throws TskCoreException {
 		synchronized (this) {
 			if (volumeSystemHandle == 0) {
-				volumeSystemHandle = SleuthkitJNI.openVs(getImage().getImageHandle(), imgOffset);
+				getVolumeSystemHandle();
 			}
 		}
 		return SleuthkitJNI.readVs(volumeSystemHandle, readBuffer, offset, len);
@@ -98,7 +98,13 @@ public class VolumeSystem extends AbstractContent {
 	 */
 	protected synchronized long getVolumeSystemHandle() throws TskCoreException {
 		if (volumeSystemHandle == 0) {
-			volumeSystemHandle = SleuthkitJNI.openVs(getImage().getImageHandle(), imgOffset);
+			Content dataSource = getDataSource();
+			if ((dataSource != null) && (dataSource instanceof Image)) {
+				Image image = (Image) dataSource;
+				volumeSystemHandle = SleuthkitJNI.openVs(image.getImageHandle(), imgOffset);
+			} else {
+				throw new TskCoreException("Volume System data source is not an image");
+			}
 		}
 
 		return volumeSystemHandle;
@@ -145,11 +151,6 @@ public class VolumeSystem extends AbstractContent {
 		return getSleuthkitCase().getVolumeSystemChildrenIds(this);
 	}
 
-	@Override
-	public Image getImage() throws TskCoreException {
-		return getParent().getImage();
-	}
-
 	/**
 	 * @return a list of Volumes that are direct children of this VolumeSystem
 	 * @throws TskCoreException
@@ -166,6 +167,6 @@ public class VolumeSystem extends AbstractContent {
 
 	@Override
 	public String toString(boolean preserveState) {
-		return super.toString(preserveState) + "VolumeSystem [\t" + "blockSize " + blockSize + "\t" + "imgOffset " + imgOffset + "\t" + "type " + type + "]\t";
+		return super.toString(preserveState) + "VolumeSystem [\t" + "blockSize " + blockSize + "\t" + "imgOffset " + imgOffset + "\t" + "type " + type + "]\t"; //NON-NLS
 	}
 }
