@@ -11,7 +11,7 @@
 
 /* Test and compare the file attribute list apis */
 
-#include "tsk3/tsk_tools_i.h"
+#include "tsk/tsk_tools_i.h"
 
 static char *s_root;
 
@@ -119,7 +119,7 @@ test_get_apis(TSK_FS_INFO * a_fs, TSK_INUM_T a_addr, int a_len)
         if (fs_attr != fs_attr2) {
             fprintf(stderr,
                 "Attribute from get_type not same addr as original %lu vs %lu from %"
-                PRIuINUM "\n", (long) fs_attr, (long) fs_attr2, a_addr);
+                PRIuINUM "\n", fs_attr, fs_attr2, a_addr);
             tsk_error_print(stderr);
             return 1;
         }
