@@ -78,6 +78,7 @@ bool opt_allocated_only = false;
 bool opt_body_file = false;
 bool opt_ignore_ntfs_system_files = false;
 bool opt_parent_tracking = false;
+bool opt_sector_hash = false;
 
 const char *config_file = 0;
 int  file_count_max = 0;
@@ -89,8 +90,6 @@ int  opt_k = 4;
 
 
 u_int	sectorhash_size=512;
-//bool	opt_compute_sector_hashes = false;
-//bool	opt_print_sector_hashes = false;
 
 namelist_t namelist;		// names of files that we want to find
 
@@ -153,6 +152,7 @@ void usage()
     printf("Ways to make this program run slower:\n");
     printf("    -M = Report MD5 for each file (default on)\n");
     printf("    -1 = Report SHA1 for each file (default on)\n");
+    printf("    -S nnnn = Perform sector hashes every nnnn bytes\n");
 #ifdef HAVE_LIBMAGIC
     printf("    -f = Enable LIBMAGIC (disabled by default)");
 #else
@@ -232,7 +232,7 @@ void comment(const char *format,...)
 void partition_info(const string &name,const string &value,const string &attribute)
 {
 
-    if(name.find(" ")<0) err(1,"partition_info(%s) has a space in it",cstr(name));
+    if(name.find(" ")!=string::npos) err(1,"partition_info(%s) has a space in it",cstr(name));
     if(a) a->add_comment(name + ": " + value);
     if(t && !opt_body_file) fputs(cstr(name + ": " + value + "\n"),t);
     if(x) x->xmlout(name,value,attribute,true);
@@ -276,7 +276,7 @@ void file_info(const string &name,const string &value)
 {
     if(a) a->add_value(name,value); 
     if(t && !opt_body_file) fputs(cstr(name + ": " + value + "\n"),t); 
-    if(x) x->xmlout(name,value); 
+    if(x) x->xmlout(name,value,std::string(),true); // escape the XML
 }
 
 /* this file_info is for sending through a hash. */
@@ -497,9 +497,9 @@ int main(int argc, char * const *argv1)
 	argv = (TSK_TCHAR * const*) argv1;
 #endif
 	
-    while ((ch = GETOPT(argc, argv, _TSK_T("A:a:C:dfG:gmv1IMX:T:VZn:c:b:xOzh?"))) > 0 ) { // s: removed
+    while ((ch = GETOPT(argc, argv, _TSK_T("A:a:C:dfG:gmv1IMX:S:T:VZn:c:b:xOzh?"))) > 0 ) { // s: removed
 	switch (ch) {
-	case _TSK_T('1'): opt_sha1++;break;
+	case _TSK_T('1'): opt_sha1 = true;break;
 	case _TSK_T('m'):
 	    opt_body_file = 1;
 	    opt_sha1 = 0;
@@ -516,19 +516,17 @@ int main(int argc, char * const *argv1)
 		break;
 	case _TSK_T('C'): file_count_max = TATOI(OPTARG);break;
 	case _TSK_T('d'): opt_debug++; break;
-//	case _TSK_T('E'):
-//	    opt_print_sector_hashes = true;
-//	    opt_compute_sector_hashes=true;
-//	    break;
 	case _TSK_T('f'): opt_magic = true;break;
 	case _TSK_T('g'): opt_no_data = true; break;
   case _TSK_T('b'): opt_get_fragments = false; break;
 	case _TSK_T('G'): opt_maxgig = TATOI(OPTARG);break;
 	case _TSK_T('h'): usage(); break;
 	case _TSK_T('I'): opt_ignore_ntfs_system_files=true;break;
-	case _TSK_T('M'): opt_md5++; break;
+	case _TSK_T('M'): opt_md5 = true;
 	case _TSK_T('O'): opt_allocated_only=true; break;
-//	case _TSK_T('S'): sectorhash_size = TATOI(OPTARG); break;
+	case _TSK_T('S'):
+            opt_sector_hash = true;
+            sectorhash_size = TATOI(OPTARG); break;
 	case _TSK_T('T'):
 #ifdef TSK_WIN32
 		convert(OPTARG, &opt_arg);
