@@ -25,6 +25,8 @@
 #include "unicode_escape.h"
 #include "tsk/fs/tsk_fatfs.h"
 
+#define MAX_SPARSE_SIZE 1024*1024*64
+
 #ifdef _MSC_VER
 #define _CRT_SECURE_NO_WARNINGS
 #endif
@@ -65,78 +67,7 @@ file_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr, char *buf,
 	 size_t size, TSK_FS_BLOCK_FLAG_ENUM flags, void *ptr)
 {
     content *ci = (content *)ptr;
-
-    if(opt_debug>1){
-	printf("file_act(fs_file=%p,addr=%"PRIuDADDR" buf=%p size=%d)\n",
-	       fs_file,addr,buf,(int)size);
-	if(opt_debug>1 && ci->segs.size()==0){
-	    if(fwrite(buf,size,1,stdout)!=1) err(1,"fwrite");
-	    printf("\n");
-	}
-    }
-
-    if(size==0)  return TSK_WALK_CONT;	// can't do much with this...
-
-    if(opt_no_data==false){
-	if (flags & TSK_FS_BLOCK_FLAG_SPARSE){
-	    /* hash zeros */
-	    ci->set_invalid(true);		// make this data set invalid
-	}
-	else {
-	    ci->add_bytes(buf,a_off,size);	// add these bytes to the file
-	}
-    }
-
-    /* "Address 0 is reserved in ExtX and FFS to denote a "sparse"
-       block (one which is all zeros).  TSK knows this and returns
-       zeros when a file refers to block 0.  You can check the 'flags'
-       argument to the callback to determine if the data is from
-       sparse or compressed data. RAW means that the data in the
-       buffer was read from the disk.
-
-       TSK_FS_BLOCK_FLAG_RAW - data on the disk
-       TSK_FS_BLOCK_FLAG_SPARSE - a whole
-       TSK_FS_BLOCK_FLAG_COMP - the file is compressed
-    */
-
-    uint64_t  fs_offset = (addr)*fs_file->fs_info->block_size;
-    uint64_t img_offset = current_partition_start + fs_offset;
-
-    if(ci->segs.size()>0){
-	/* Does this next segment fit after the prevous segment logically? */
-	if(ci->segs.back().next_file_offset()==(uint64_t)a_off){
-
-	    /* if both the last and the current are sparse, this can be extended. */
-	    if((ci->segs.back().flags & TSK_FS_BLOCK_FLAG_SPARSE) &&
-	       (flags & TSK_FS_BLOCK_FLAG_SPARSE)){
-
-		ci->segs.back().len += size;
-		return TSK_WALK_CONT;
-	    }
-
-
-	    /* If both are compressed, then this can be extended? */
-	    if((ci->segs.back().flags & TSK_FS_BLOCK_FLAG_COMP) &&
-	       (flags & TSK_FS_BLOCK_FLAG_COMP) &&
-	       (ci->segs.back().img_offset + ci->segs.back().len == img_offset)){
-		ci->segs.back().len += size;
-		return TSK_WALK_CONT;
-	    }
-
-	    /* See if we can extend the last segment in the segment list,
-	     * or if this is the start of a new fragment.
-	     */
-	    if((ci->segs.back().flags & TSK_FS_BLOCK_FLAG_RAW) &&
-	       (flags & TSK_FS_BLOCK_FLAG_RAW) &&
-	       (ci->segs.back().img_offset + ci->segs.back().len == img_offset)){
-		ci->segs.back().len += size;
-		return TSK_WALK_CONT;
-	    }
-	}
-    }
-    /* Need to add a new element to the list */
-    ci->add_seg(img_offset,fs_offset,(int64_t)a_off,size,flags);
-    return TSK_WALK_CONT;
+    return ci->file_act(fs_file,a_off,addr,buf,size,flags);
 }
 
 /* This is modeled on print_dent_act printit in ./tsk/fs/fls_lib.c
@@ -146,6 +77,9 @@ file_act(TSK_FS_FILE * fs_file, TSK_OFF_T a_off, TSK_DADDR_T addr, char *buf,
 static uint8_t
 process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
 {
+    /* Use a flag to determine if a file is generically fit for plugins. */
+    bool can_run_plugin;
+
     /* Make sure that the SleuthKit structures are properly set */
     if (fs_file->name == NULL) 
         return 1;
@@ -329,24 +263,21 @@ process_tsk_file(TSK_FS_FILE * fs_file, const char *path)
     ci.write_record();
 
 
-    /* Processing for regular files: */
+    /* Processing for regular files and some virtual files: */
+    can_run_plugin = false;
     if(fs_file->name->type == TSK_FS_NAME_TYPE_REG){
-
-	if(ci.do_plugin && ci.total_bytes>0) plugin_process(ci.tempfile_path);
-
-	/* Output the sector hashes to text or XML file if requested */
-//	if(opt_compute_sector_hashes){
-//	    int count = 1;
-//	    for(vector<string>::const_iterator it = ci.sectorhashes.begin();
-//		it!=ci.sectorhashes.end(); it++){
-//		if(opt_print_sector_hashes){
-//		    if(t) fprintf(t,"sectorhash: %s %d\n",(*it).c_str(),count);
-//		    if(x) x->xmlout("sectorhash",*it,"",false);
-//		}
-//		count++;
-//	    }
-//	}
+        can_run_plugin = true;
     }
+    else if(fs_file->name->type == TSK_FS_NAME_TYPE_VIRT){
+        /* Pass some virtual files to plugins, e.g. $MBR for boot sector virus scans. */
+        if(fs_file->name->name){
+            if(strcmp(fs_file->name->name, "$MBR") == 0) {
+                can_run_plugin = true;
+            }
+        }
+    }
+
+    if(can_run_plugin && ci.do_plugin && ci.total_bytes>0) plugin_process(ci.tempfile_path);
 
     /* END of file processing */
     if(x) x->pop();
@@ -365,7 +296,7 @@ dir_act(TSK_FS_FILE * fs_file, const char *path, void *ptr)
 {
     /* Ignore NTFS System files */
     if (opt_ignore_ntfs_system_files
-	&& (TSK_FS_TYPE_ISNTFS(fs_file->fs_info->ftype))
+	&& (TSK_FS_TYPE_ISNTFS(fs_file->fs_info->ftype) || TSK_FS_TYPE_ISFAT(fs_file->fs_info->ftype))
         && (fs_file->name->name[0] == '$'))
         return TSK_WALK_CONT;
 
@@ -556,7 +487,7 @@ void process_scalpel_audit_file(TSK_IMG_INFO *img_info,const char *audit_file)
 		    file_info("carvelength",length);
 		}
 
-		ci.add_seg(start,start,0,r2,TSK_FS_BLOCK_FLAG_RAW);	// may not be able to read it all
+		ci.add_seg(start,start,0,r2,TSK_FS_BLOCK_FLAG_RAW,"");	// may not be able to read it all
 		ci.add_bytes(buf2,0,r2);
 		ci.write_record();
 		free(buf2);
