@@ -134,13 +134,17 @@ make_pt_tls_key()
 TSK_ERROR_INFO *
 tsk_error_get_info()
 {
-    TSK_ERROR_INFO *ptr = 0;
+    TSK_ERROR_INFO *ptr = NULL;
     (void) pthread_once(&pt_tls_key_once, make_pt_tls_key);
     if ((ptr = (TSK_ERROR_INFO *) pthread_getspecific(pt_tls_key)) == 0) {
+        // Under high memory presure malloc will return NULL.
         ptr = (TSK_ERROR_INFO *) malloc(sizeof(TSK_ERROR_INFO));
-        ptr->t_errno = 0;
-        ptr->errstr[0] = 0;
-        ptr->errstr2[0] = 0;
+
+        if( ptr != NULL ) {
+            ptr->t_errno = 0;
+            ptr->errstr[0] = 0;
+            ptr->errstr2[0] = 0;
+        }
         (void) pthread_setspecific(pt_tls_key, ptr);
     }
     return ptr;
@@ -302,7 +306,7 @@ tsk_error_get_errstr()
  * @param format the printf-style format string
  */
 void
-tsk_error_set_errstr(char const *format, ...)
+tsk_error_set_errstr(const char *format, ...)
 {
     va_list args;
     va_start(args, format);
@@ -318,7 +322,7 @@ tsk_error_set_errstr(char const *format, ...)
  * @param args the printf-style args
  */
 void
-tsk_error_vset_errstr(char const *format, va_list args)
+tsk_error_vset_errstr(const char *format, va_list args)
 {
     vsnprintf(tsk_error_get_info()->errstr, TSK_ERROR_STRING_MAX_LENGTH,
         format, args);
@@ -343,7 +347,7 @@ tsk_error_get_errstr2()
  * @param format the printf-style format string
  */
 void
-tsk_error_set_errstr2(char const *format, ...)
+tsk_error_set_errstr2(const char *format, ...)
 {
     va_list args;
     va_start(args, format);
@@ -359,7 +363,7 @@ tsk_error_set_errstr2(char const *format, ...)
  * @param args the printf-style format args
  */
 void
-tsk_error_vset_errstr2(char const *format, va_list args)
+tsk_error_vset_errstr2(const char *format, va_list args)
 {
     vsnprintf(tsk_error_get_info()->errstr2, TSK_ERROR_STRING_MAX_LENGTH,
         format, args);
@@ -371,7 +375,7 @@ tsk_error_vset_errstr2(char const *format, va_list args)
  * @param format
  */
 void
-tsk_error_errstr2_concat(char const *format, ...)
+tsk_error_errstr2_concat(const char *format, ...)
 {
     va_list args;
     char *errstr2 = tsk_error_get_info()->errstr2;
@@ -417,8 +421,11 @@ void
 tsk_error_reset()
 {
     TSK_ERROR_INFO *info = tsk_error_get_info();
-    info->t_errno = 0;
-    info->errstr[0] = 0;
-    info->errstr2[0] = 0;
-    info->errstr_print[0] = 0;
+
+    if( info != NULL ) {
+       info->t_errno = 0;
+       info->errstr[0] = 0;
+       info->errstr2[0] = 0;
+       info->errstr_print[0] = 0;
+    }
 }
