@@ -51,6 +51,7 @@ printit(TSK_FS_FILE * fs_file, const char *a_path,
     const TSK_FS_ATTR * fs_attr, const FLS_DATA * fls_data)
 {
     TSK_FS_HASH_RESULTS hash_results;
+	unsigned char null_buf[16];
     unsigned int i;
 
     if ((!(fls_data->flags & TSK_FS_FLS_FULL)) && (a_path)) {
@@ -69,11 +70,19 @@ printit(TSK_FS_FILE * fs_file, const char *a_path,
 
     if (fls_data->flags & TSK_FS_FLS_MAC) {
         if (fls_data->flags & TSK_FS_FLS_HASH) {
-            tsk_fs_file_hash_calc(fs_file, &hash_results,
-                TSK_BASE_HASH_MD5);
-            tsk_fs_name_print_mac_md5(stdout, fs_file, a_path, fs_attr,
-                fls_data->macpre, fls_data->sec_skew,
-                hash_results.md5_digest);
+            if(0 == tsk_fs_file_hash_calc(fs_file, &hash_results,
+                TSK_BASE_HASH_MD5)){
+				tsk_fs_name_print_mac_md5(stdout, fs_file, a_path, fs_attr,
+					fls_data->macpre, fls_data->sec_skew,
+					hash_results.md5_digest);
+			}
+			else{
+				// If the hash calculation had errors, pass in a buffer of nulls
+				memset(null_buf, 0, 16);
+				tsk_fs_name_print_mac_md5(stdout, fs_file, a_path, fs_attr,
+					fls_data->macpre, fls_data->sec_skew,
+					null_buf);
+			}
         }
         else {
             tsk_fs_name_print_mac(stdout, fs_file, a_path,
