@@ -7,7 +7,7 @@
 ** depending on the file system type.
 **
 ** Brian Carrier [carrier <at> sleuthkit [dot] org]
-** Copyright (c) 2006-2011 Brian Carrier.  All Rights reserved
+** Copyright (c) 2006-2013 Brian Carrier.  All Rights reserved
 ** Copyright (c) 2003-2005 Brian Carrier.  All rights reserved 
 **
 ** TASK
@@ -111,6 +111,7 @@ tsk_fs_name_reset(TSK_FS_NAME * a_fs_name)
     a_fs_name->meta_addr = 0;
     a_fs_name->meta_seq = 0;
     a_fs_name->par_addr = 0;
+    a_fs_name->par_seq = 0;
     a_fs_name->type = 0;
     a_fs_name->flags = 0;
 }
@@ -196,6 +197,7 @@ tsk_fs_name_copy(TSK_FS_NAME * a_fs_name_to,
     a_fs_name_to->meta_addr = a_fs_name_from->meta_addr;
     a_fs_name_to->meta_seq = a_fs_name_from->meta_seq;
     a_fs_name_to->par_addr = a_fs_name_from->par_addr;
+    a_fs_name_to->par_seq = a_fs_name_from->par_seq;
     a_fs_name_to->type = a_fs_name_from->type;
     a_fs_name_to->flags = a_fs_name_from->flags;
 
@@ -395,6 +397,7 @@ tsk_fs_name_print(FILE * hFile, const TSK_FS_FILE * fs_file,
     uint8_t print_path)
 {
     size_t i;
+    char *buf;
 
     /* type of file - based on dentry type */
     if (fs_file->name->type < TSK_FS_NAME_TYPE_STR_MAX)
@@ -449,37 +452,40 @@ tsk_fs_name_print(FILE * hFile, const TSK_FS_FILE * fs_file,
                 flags & TSK_FS_NAME_FLAG_UNALLOC)) ? "(realloc)" : "");
 
     if ((print_path) && (a_path != NULL)) {
-        for (i = 0; i < strlen(a_path); i++) {
-            if (TSK_IS_CNTRL(a_path[i]))
-                tsk_fprintf(hFile, "^");
-            else
-                tsk_fprintf(hFile, "%c", a_path[i]);
+        buf = malloc(strlen(a_path) + 1);
+        strcpy(buf, a_path);
+        for (i = 0; i < strlen(buf); i++) {
+            if (TSK_IS_CNTRL(buf[i])) buf[i] = '^';
         }
+        tsk_fprintf(hFile, "%s", buf);
+        free(buf);
     }
 
-    for (i = 0; i < strlen(fs_file->name->name); i++) {
-        if (TSK_IS_CNTRL(fs_file->name->name[i]))
-            tsk_fprintf(hFile, "^");
-        else
-            tsk_fprintf(hFile, "%c", fs_file->name->name[i]);
+    buf = malloc(strlen(fs_file->name->name) + 1);
+    strcpy(buf, fs_file->name->name);
+    for (i = 0; i < strlen(buf); i++) {
+        if (TSK_IS_CNTRL(buf[i])) buf[i] = '^';
     }
+    tsk_fprintf(hFile, "%s", buf);
+    free(buf);
 
-/*  This will add the short name in parentheses
-    if (fs_file->name->shrt_name != NULL && fs_file->name->shrt_name[0] != '\0')
-	tsk_fprintf(hFile, " (%s)", fs_file->name->shrt_name);
-*/
+    /*  This will add the short name in parentheses
+        if (fs_file->name->shrt_name != NULL && fs_file->name->shrt_name[0] != '\0')
+        tsk_fprintf(hFile, " (%s)", fs_file->name->shrt_name);
+        */
 
     /* print the data stream name if we the non-data NTFS stream */
     if ((fs_attr) && (fs_attr->name)) {
         if ((fs_attr->type != TSK_FS_ATTR_TYPE_NTFS_IDXROOT) ||
             (strcmp(fs_attr->name, "$I30") != 0)) {
             tsk_fprintf(hFile, ":");
-            for (i = 0; i < strlen(fs_attr->name); i++) {
-                if (TSK_IS_CNTRL(fs_attr->name[i]))
-                    tsk_fprintf(hFile, "^");
-                else
-                    tsk_fprintf(hFile, "%c", fs_attr->name[i]);
+            buf = malloc(strlen(fs_attr->name) + 1);
+            strcpy(buf, fs_attr->name);
+            for (i = 0; i < strlen(buf); i++) {
+                if (TSK_IS_CNTRL(buf[i])) buf[i] = '^';
             }
+            tsk_fprintf(hFile, "%s", buf);
+            free(buf);
         }
     }
 
@@ -558,6 +564,8 @@ tsk_fs_name_print_long(FILE * hFile, const TSK_FS_FILE * fs_file,
 }
 
 
+
+
 /**
  * \internal
  *
@@ -579,47 +587,102 @@ void
 tsk_fs_name_print_mac(FILE * hFile, const TSK_FS_FILE * fs_file,
     const char *a_path, const TSK_FS_ATTR * fs_attr,
     const char *prefix, int32_t time_skew)
+{
+    tsk_fs_name_print_mac_md5(hFile, fs_file, a_path, fs_attr, prefix,
+        time_skew, NULL);
+}
+
+/**
+ * \internal
+ *
+** Print output in the format that mactime reads.
+**
+** If the flags in the fs_file->meta structure are set to FS_FLAG_ALLOC
+** then it is assumed that the inode has been reallocated and the
+** contents are not displayed
+**
+** fs is not required (only used for block size).
+ * @param hFile handle to print results to
+ * @param fs_file File to print details about
+ * @param a_path Parent directory of file (needs to end with "/")
+ * @param fs_attr Attribute in file that is being called for (NULL for non-NTFS)
+ * @param prefix Path of mounting point for image
+ * @param time_skew number of seconds skew to adjust time
+ * @param hash_results Holds the calculated md5 hash
+*/
+void
+tsk_fs_name_print_mac_md5(FILE * hFile, const TSK_FS_FILE * fs_file,
+    const char *a_path, const TSK_FS_ATTR * fs_attr,
+    const char *prefix, int32_t time_skew,
+    const unsigned char *hash_results)
 {
     char ls[12];
     size_t i;
+    uint8_t isADS = 0;
+    char *buf;
 
     if ((!hFile) || (!fs_file))
         return;
 
-    /* md5 */
-    tsk_fprintf(hFile, "0|");
+    /* see if we are going to be printing the name of the attribute
+     * We don't do it for FNAME attributes, which we handle specially below.
+     */
+    if ((fs_attr) && (fs_attr->name)
+        && (fs_attr->type != TSK_FS_ATTR_TYPE_NTFS_FNAME)
+        && ((fs_attr->type != TSK_FS_ATTR_TYPE_NTFS_IDXROOT)
+            || (strcmp(fs_attr->name, "$I30") != 0))) {
+        isADS = 1;
+    }
+
+    /* hash
+     * Print out the hash buffer (if not null)
+     */
+    if (hash_results == NULL) {
+        tsk_fprintf(hFile, "0|");
+    }
+    else {
+        for (i = 0; i < 16; i++) {
+            tsk_fprintf(hFile, "%02x", hash_results[i]);
+        }
+        tsk_fprintf(hFile, "|");
+    }
 
     /* file name */
     tsk_fprintf(hFile, "%s", prefix);
 
     // remove any control chars as we print the names
     if (a_path != NULL) {
-        for (i = 0; i < strlen(a_path); i++) {
-            if (TSK_IS_CNTRL(a_path[i]))
-                tsk_fprintf(hFile, "^");
-            else
-                tsk_fprintf(hFile, "%c", a_path[i]);
+        buf = malloc(strlen(a_path) + 1);
+        strcpy(buf, a_path);
+        for (i = 0; i < strlen(buf); i++) {
+            if (TSK_IS_CNTRL(buf[i])) buf[i] = '^';
         }
+        tsk_fprintf(hFile, "%s", buf);
+        free(buf);
     }
-
-    for (i = 0; i < strlen(fs_file->name->name); i++) {
-        if (TSK_IS_CNTRL(fs_file->name->name[i]))
-            tsk_fprintf(hFile, "^");
-        else
-            tsk_fprintf(hFile, "%c", fs_file->name->name[i]);
+    buf = malloc(strlen(fs_file->name->name) + 1);
+    strcpy(buf, fs_file->name->name);
+    for (i = 0; i < strlen(buf); i++) {
+        if (TSK_IS_CNTRL(buf[i])) buf[i] = '^';
     }
+    tsk_fprintf(hFile, "%s", buf);
+    free(buf);
 
     /* print the data stream name if it exists and is not the default NTFS */
-    if ((fs_attr) && (fs_attr->name) &&
-        ((fs_attr->type != TSK_FS_ATTR_TYPE_NTFS_IDXROOT) ||
-            (strcmp(fs_attr->name, "$I30") != 0))) {
+    if (isADS) {
         tsk_fprintf(hFile, ":");
-        for (i = 0; i < strlen(fs_attr->name); i++) {
-            if (TSK_IS_CNTRL(fs_attr->name[i]))
-                tsk_fprintf(hFile, "^");
-            else
-                tsk_fprintf(hFile, "%c", fs_attr->name[i]);
+        buf = malloc(strlen(fs_attr->name) + 1);
+        strcpy(buf, fs_attr->name);
+        for (i = 0; i < strlen(buf); i++) {
+            if (TSK_IS_CNTRL(buf[i])) buf[i] = '^';
         }
+        tsk_fprintf(hFile, "%s", buf);
+        free(buf);
+    }
+
+    // special label if FNAME
+    if ((fs_attr) && (fs_attr->type == TSK_FS_ATTR_TYPE_NTFS_FNAME)) {
+        tsk_fprintf(hFile, " ($FILE_NAME)");
     }
 
     if ((fs_file->meta)
@@ -643,7 +706,7 @@ tsk_fs_name_print_mac(FILE * hFile, const TSK_FS_FILE * fs_file,
 
     tsk_fprintf(hFile, "|");
 
-    /* TYPE as specified in the directory entry 
+    /* TYPE as specified in the directory entry
      */
     if (fs_file->name->type < TSK_FS_NAME_TYPE_STR_MAX)
         tsk_fprintf(hFile, "%s/",
@@ -652,10 +715,9 @@ tsk_fs_name_print_mac(FILE * hFile, const TSK_FS_FILE * fs_file,
         tsk_fprintf(hFile, "-/");
 
     if (!fs_file->meta) {
-        tsk_fprintf(hFile, "----------|0|0|0|0|0|0|0\n");
+        tsk_fprintf(hFile, "----------|0|0|0|");
     }
     else {
-
         /* mode as string */
         tsk_fs_meta_make_ls(fs_file->meta, ls, sizeof(ls));
         tsk_fprintf(hFile, "%s|", ls);
@@ -669,30 +731,68 @@ tsk_fs_name_print_mac(FILE * hFile, const TSK_FS_FILE * fs_file,
             tsk_fprintf(hFile, "%" PRIuOFF "|", fs_attr->size);
         else
             tsk_fprintf(hFile, "%" PRIuOFF "|", fs_file->meta->size);
+    }
 
-        /* atime, mtime, ctime, crtime */
-        if (fs_file->meta->atime)
-            tsk_fprintf(hFile, "%" PRIu32 "|",
-                fs_file->meta->atime - time_skew);
-        else
-            tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->atime);
+    if (!fs_file->meta) {
+        tsk_fprintf(hFile, "0|0|0|0\n");
+    }
+    else {
+        // special case for NTFS FILE_NAME attribute
+        if ((fs_attr) && (fs_attr->type == TSK_FS_ATTR_TYPE_NTFS_FNAME)) {
+            /* atime, mtime, ctime, crtime */
+            if (fs_file->meta->time2.ntfs.fn_atime)
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->time2.ntfs.fn_atime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->time2.ntfs.fn_atime);
 
-        if (fs_file->meta->mtime)
-            tsk_fprintf(hFile, "%" PRIu32 "|",
-                fs_file->meta->mtime - time_skew);
-        else
-            tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->mtime);
+            if (fs_file->meta->time2.ntfs.fn_mtime)
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->time2.ntfs.fn_mtime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->time2.ntfs.fn_mtime);
 
-        if (fs_file->meta->ctime)
-            tsk_fprintf(hFile, "%" PRIu32 "|",
-                fs_file->meta->ctime - time_skew);
-        else
-            tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->ctime);
+            if (fs_file->meta->time2.ntfs.fn_ctime)
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->time2.ntfs.fn_ctime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->time2.ntfs.fn_ctime);
 
-        if (fs_file->meta->crtime)
-            tsk_fprintf(hFile, "%" PRIu32 "\n",
-                fs_file->meta->crtime - time_skew);
-        else
-            tsk_fprintf(hFile, "%" PRIu32 "\n", fs_file->meta->crtime);
+            if (fs_file->meta->time2.ntfs.fn_crtime)
+                tsk_fprintf(hFile, "%" PRIu32 "\n",
+                    fs_file->meta->time2.ntfs.fn_crtime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "\n",
+                    fs_file->meta->time2.ntfs.fn_crtime);
+        }
+        else {
+            /* atime, mtime, ctime, crtime */
+            if (fs_file->meta->atime)
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->atime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->atime);
+
+            if (fs_file->meta->mtime)
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->mtime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->mtime);
+
+            if (fs_file->meta->ctime)
+                tsk_fprintf(hFile, "%" PRIu32 "|",
+                    fs_file->meta->ctime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "|", fs_file->meta->ctime);
+
+            if (fs_file->meta->crtime)
+                tsk_fprintf(hFile, "%" PRIu32 "\n",
+                    fs_file->meta->crtime - time_skew);
+            else
+                tsk_fprintf(hFile, "%" PRIu32 "\n", fs_file->meta->crtime);
+        }
     }
 }
