@@ -239,7 +239,7 @@ hfs_dir_open_meta_cb(HFS_INFO * hfs, int8_t level_type,
            if (rec_off2 > nodesize) {
            tsk_error_set_errno(TSK_ERR_FS_GENFS);
            tsk_error_set_errstr(
-           "hfs_dir_open_meta: offset of record+keylen %d in leaf node %d too large (%zu vs %"
+           "hfs_dir_open_meta: offset of record+keylen %d in leaf node %d too large (%"PRIuSIZE" vs %"
            PRIu16 ")", rec, cur_node, rec_off2, nodesize);
            tsk_fs_name_free(fs_name);
            free(node);
