@@ -1,16 +1,16 @@
 /*
 ** The Sleuth Kit
 **
-** $Date: 2007/12/20 16:18:10 $
-**
-** Brian Carrier [carrier@sleuthkit.org]
-** Copyright (c) 2003-2008 Brian Carrier.  All rights reserved
+** Brian Carrier [carrier <at> sleuthkit [dot] org]
+** Copyright (c) 2003-2011 Brian Carrier.  All rights reserved
 **
 ** TASK
 ** Copyright (c) 2002 @stake Inc.  All rights reserved
+**
+** This software is distributed under the Common Public License 1.0
 */
 
-/** \file tsk_ntfs.h
+/*
  * Contains the structures and function APIs for NTFS file system support.
  */
 
@@ -25,11 +25,10 @@ extern "C" {
 
 // the SID code has been buggy on some systems and byitself it does
 // not provide much security info.  It is being disabled until fixed. 
-#define TSK_USE_SID 0
+#define TSK_USE_SID 1
 
 //#define NTFS_FS_MAGIC 0x5346544E      /* "NTFS" in little endian */
 #define NTFS_FS_MAGIC	0xAA55
-#define NTFS_DEV_BSIZE	512
 
 #define NTFS_MAXNAMLEN	256
 #define NTFS_MAXNAMLEN_UTF8	4 * NTFS_MAXNAMLEN
@@ -39,8 +38,7 @@ extern "C" {
 #define NTFS_FIRSTINO	0       /* location of the $Mft Record */
 #define NTFS_LAST_DEFAULT_INO	16      /* A guess for right now */
 
-#define NTFS_NDADDR			0
-#define NTFS_NIADDR			0
+#define NTFS_FILE_CONTENT_LEN 0
 
 /* uncompression values */
 #define NTFS_TOKEN_MASK   1
@@ -186,6 +184,8 @@ extern "C" {
 
 
 /* values for the type field */
+/* NOTE that the default TSK attribute types are based on these.
+ * Any changes to these should be merged with the defines in tsk_fs.h */
 #define NTFS_ATYPE_SI       0x10        // 16
 #define NTFS_ATYPE_ATTRLIST 0x20        // 32
 #define NTFS_ATYPE_FNAME    0x30        // 48
@@ -523,9 +523,9 @@ extern "C" {
  * Structure used in Security Descriptor lookups
  */
     typedef struct {
-        char *buffer;
-        size_t size;
-        size_t used;
+        char *buffer;           ///< Buffer to store data in
+        size_t size;            ///< Number of bytes in buffer
+        size_t used;            ///< Number of records used in the buffer (size depends on type of data stored)
     } NTFS_SXX_BUFFER;
 
 
@@ -541,29 +541,6 @@ extern "C" {
         uint32_t sub_auth[1];   /* At least one sub_auth */
     } ntfs_sid;
 
-    void ntfs_print_sid(TSK_FS_INFO * fs, FILE * hFile);
-
-    char *ntfs_get_sid_as_string(TSK_FS_INFO * fs, uint32_t security_id);
-
-
-
-#define NTFS_ACE_SIZE	16      /* Size in bytes of ACE information proceeding
-
-                                   the SID.  Since we don't current require
-
-                                   the ACE information, we will use this as an
-
-                                   offset to get to the start of the SID */
-
-    typedef struct NTFS_SID_ENTRY NTFS_SID_ENTRY;
-
-    struct NTFS_SID_ENTRY {
-        NTFS_SID_ENTRY *next;
-        ntfs_sid *data;         /* ntfs_sid record */
-        uint32_t sec_id;        /* Security ID */
-        char *sid_str;          /* SID string representation */
-    };
-
 
 
 /************************************************************************
@@ -579,18 +556,6 @@ extern "C" {
     } ntfs_attr_sds;
 
 
-#define NTFS_SDS_BLOCK_OFFSET	262144  /* Data offset within $SDS (1024x256) */
-
-    typedef struct NTFS_SDS_ENTRY NTFS_SDS_ENTRY;
-
-    struct NTFS_SDS_ENTRY {
-        NTFS_SDS_ENTRY *next;
-        uint32_t len;           /* Length of data */
-        uint8_t *data;          /* Raw data */
-    };
-
-
-
 /************************************************************************
  * SDH attribute
  */
@@ -613,14 +578,6 @@ extern "C" {
         uint8_t pad3[4];        /* Padding */
     } ntfs_attr_sdh;
 
-    typedef struct NTFS_SDH_ENTRY NTFS_SDH_ENTRY;
-
-    struct NTFS_SDH_ENTRY {
-        NTFS_SDH_ENTRY *next;
-        ntfs_attr_sdh *data;    /* ntfs_attr_sdh record */
-    };
-
-
 
 /************************************************************************
  * SII attribute
@@ -642,55 +599,68 @@ extern "C" {
 
     } ntfs_attr_sii;
 
-    typedef struct NTFS_SII_ENTRY NTFS_SII_ENTRY;
 
-    struct NTFS_SII_ENTRY {
-        NTFS_SII_ENTRY *next;
-        ntfs_attr_sii *data;    /* ntfs_attr_sii record */
-    };
 #endif
 
+
+
+
+
 /************************************************************************
 */
     typedef struct {
         TSK_FS_INFO fs_info;    /* super class */
         ntfs_sb *fs;
         uint8_t ver;            /* version of NTFS - uses the VINFO flag */
-        TSK_FS_INODE *mft_inode;        /* contains the data for the mft entry for the mft */
-        TSK_FS_DATA *mft_data;  /* Data run for MFT entry for MFT */
-        ntfs_mft *mft;          /* cache for on-disk inode */
-        TSK_INUM_T mnum;            /* number of above cached mft */
+        TSK_FS_FILE *mft_file;  /* contains the data for the mft entry for the mft */
+        const TSK_FS_ATTR *mft_data;    /* Data run for MFT entry for MFT */
         uint32_t csize_b;       /* number of bytes in a cluster */
         uint16_t ssize_b;       /* number of bytes in a sector */
         uint32_t mft_rsize_b;   /* number of bytes per mft record */
         uint32_t idx_rsize_b;   /* number of bytes per idx record */
-        TSK_DADDR_T root_mft_addr;  /* address of first mft entry */
+        TSK_DADDR_T root_mft_addr;      /* address of first mft entry */
 
         uint8_t loading_the_MFT;        /* set to 1 when initializing the setup */
 
-        TSK_FS_DATA_RUN *bmap;  /* Run of bitmap for clusters (linked list) */
-        TSK_DATA_BUF *bmap_buf; /* buffer to hold cached copy of bitmap */
-        TSK_DADDR_T bmap_buf_off;   /* offset cluster in cached bitmap */
+        TSK_FS_ATTR_RUN *bmap;  /* Run of bitmap for clusters (linked list) */
+
+        /* lock protects bmap_buf, bmap_buf_off */
+        tsk_lock_t lock;
+        char *bmap_buf;         /* buffer to hold cached copy of bitmap (r/w shared - lock)  */
+        TSK_DADDR_T bmap_buf_off;       /* offset cluster in cached bitmap  (r/w shared - lock) */
+
         ntfs_attrdef *attrdef;  // buffer of attrdef file contents
-        size_t attrdef_len;    // length of addrdef buffer
+        size_t attrdef_len;     // length of addrdef buffer
+
+        /* orphan_map_lock protects orphan_map */
+        tsk_lock_t orphan_map_lock;
+        void *orphan_map;       // map that lists par directory to its orphans. (r/w shared - lock) 
 
 #if TSK_USE_SID
-        NTFS_SDS_ENTRY *sds;    /* Data run of ntfs_attr_sds */
-        //NTFS_SDH_ENTRY *sdh;  /* Data run of ntfs_attr_sdh */
-        //NTFS_SII_ENTRY *sii;  /* Data run of ntfs_attr_sii */
-        NTFS_SID_ENTRY *sid;    /* Data run of ntfs_sid */
+        /* sid_lock protects sii_data, sds_data */
+        tsk_lock_t sid_lock;
+        NTFS_SXX_BUFFER sii_data;       // (r/w shared - lock) 
+        NTFS_SXX_BUFFER sds_data;       // (r/w shared - lock) 
 #endif
     } NTFS_INFO;
 
-    extern uint8_t
-        ntfs_data_walk(NTFS_INFO *, TSK_INUM_T, TSK_FS_DATA *, int,
-        TSK_FS_FILE_WALK_CB, void *);
-    extern uint8_t
-        ntfs_dent_walk(TSK_FS_INFO *, TSK_INUM_T, TSK_FS_DENT_FLAG_ENUM,
-        TSK_FS_DENT_TYPE_WALK_CB, void *);
+
     extern uint32_t nt2unixtime(uint64_t ntdate);
     extern uint8_t ntfs_attrname_lookup(TSK_FS_INFO *, uint16_t, char *,
         int);
+    extern TSK_RETVAL_ENUM ntfs_dinode_lookup(NTFS_INFO *, char *,
+        TSK_INUM_T);
+    extern TSK_RETVAL_ENUM ntfs_dir_open_meta(TSK_FS_INFO * a_fs,
+        TSK_FS_DIR ** a_fs_dir, TSK_INUM_T a_addr);
+
+    extern void ntfs_orphan_map_free(NTFS_INFO * a_ntfs);
+
+    extern int ntfs_name_cmp(TSK_FS_INFO *, const char *, const char *);
+
+    extern uint8_t ntfs_find_file(TSK_FS_INFO * fs, TSK_INUM_T inode_toid,
+        uint32_t type_toid, uint8_t type_used, uint16_t id_toid,
+        uint8_t id_used, TSK_FS_DIR_WALK_FLAG_ENUM dir_walk_flags,
+        TSK_FS_DIR_WALK_CB action, void *ptr);
 
 #ifdef __cplusplus
 }
