@@ -1,15 +1,13 @@
 /*
- * $Date: 2007/12/19 23:12:10 $
- *
- * Brian Carrier [carrier@sleuthkit.org]
- * Copyright (c) 2006-2007 Brian Carrier, Basis Technology.  All rights reserved
+ * Brian Carrier [carrier <at> sleuthkit [dot] org]
+ * Copyright (c) 2006-2011 Brian Carrier, Basis Technology.  All rights reserved
  *
  * This software is distributed under the Common Public License 1.0
  */
 
 /**
  * \file aff.c
- * Code to interface with afflib to read and open AFF image files
+ * Internal code to interface with afflib to read and open AFF image files
  */
 
 #include "tsk_img_i.h"
@@ -20,49 +18,45 @@ typedef int bool;
 
 #include "aff.h"
 
+/* Note: The routine -assumes- we are under a lock on &(img_info->cache_lock)) */
 static ssize_t
-aff_read_random(TSK_IMG_INFO * img_info, TSK_OFF_T vol_offset, char *buf,
-    size_t len, TSK_OFF_T offset)
+aff_read(TSK_IMG_INFO * img_info, TSK_OFF_T offset, char *buf, size_t len)
 {
     ssize_t cnt;
     IMG_AFF_INFO *aff_info = (IMG_AFF_INFO *) img_info;
-    TSK_OFF_T tot_offset = offset + vol_offset;
 
     if (tsk_verbose)
         tsk_fprintf(stderr,
-            "aff_read_random: byte offset: %" PRIuOFF " len: %" PRIuOFF
+            "aff_read: byte offset: %" PRIuOFF " len: %" PRIuOFF
             "\n", offset, len);
 
-    if (tot_offset > img_info->size) {
+    if (offset > img_info->size) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_IMG_READ_OFF;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "aff_read_random - %" PRIuOFF, tot_offset);
+        tsk_error_set_errno(TSK_ERR_IMG_READ_OFF);
+        tsk_error_set_errstr("aff_read - %" PRIuOFF, offset);
         return -1;
     }
 
-    if (aff_info->seek_pos != tot_offset) {
-        if (af_seek(aff_info->af_file, tot_offset, SEEK_SET) != tot_offset) {
+    if (aff_info->seek_pos != offset) {
+        if (af_seek(aff_info->af_file, offset, SEEK_SET) != offset) {
             tsk_error_reset();
             // @@@ ADD more specific error messages
-            tsk_errno = TSK_ERR_IMG_SEEK;
-            snprintf(tsk_errstr, TSK_ERRSTR_L,
-                "aff_read_random - %" PRIuOFF " - %s", tot_offset,
+            tsk_error_set_errno(TSK_ERR_IMG_SEEK);
+            tsk_error_set_errstr("aff_read - %" PRIuOFF " - %s", offset,
                 strerror(errno));
             return -1;
 
         }
-        aff_info->seek_pos = tot_offset;
+        aff_info->seek_pos = offset;
     }
 
     cnt = af_read(aff_info->af_file, (unsigned char *) buf, len);
     if (cnt < 0) {
         // @@@ Add more specific error message
         tsk_error_reset();
-        tsk_errno = TSK_ERR_IMG_READ;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "aff_read_random - offset: %" PRIuOFF " - len: %"
-            PRIuOFF " - %s", tot_offset, len, strerror(errno));
+        tsk_error_set_errno(TSK_ERR_IMG_READ);
+        tsk_error_set_errstr("aff_read - offset: %" PRIuOFF " - len: %"
+            PRIuSIZE " - %s", offset, len, strerror(errno));
         return -1;
     }
 
@@ -72,7 +66,7 @@ aff_read_random(TSK_IMG_INFO * img_info, TSK_OFF_T vol_offset, char *buf,
         // @@@ We could improve this if there is an AFF call
         // to see if the data exists or not
         if ((af_eof(aff_info->af_file) == 0) &&
-            (tot_offset + len < img_info->size)) {
+            (offset + len < img_info->size)) {
             memset(buf, 0, len);
             cnt = len;
         }
@@ -82,13 +76,7 @@ aff_read_random(TSK_IMG_INFO * img_info, TSK_OFF_T vol_offset, char *buf,
     return cnt;
 }
 
-TSK_OFF_T
-aff_get_size(TSK_IMG_INFO * img_info)
-{
-    return img_info->size;
-}
-
-void
+static void
 aff_imgstat(TSK_IMG_INFO * img_info, FILE * hFile)
 {
     IMG_AFF_INFO *aff_info = (IMG_AFF_INFO *) img_info;
@@ -110,12 +98,16 @@ aff_imgstat(TSK_IMG_INFO * img_info, FILE * hFile)
         tsk_fprintf(hFile, "AFM\n");
         break;
     default:
-        tsk_fprintf(hFile, "?\n");
+        tsk_fprintf(hFile, "AFFLIB (%d)\n", aff_info->type);
         break;
     }
 
     tsk_fprintf(hFile, "\nSize in bytes: %" PRIuOFF "\n", img_info->size);
 
+    // we won't have the rest of the info for the non-AFF formats.
+    if (img_info->itype == TSK_IMG_TYPE_AFF_ANY)
+        return;
+
     tsk_fprintf(hFile, "\nMD5: ");
     if (af_get_seg(aff_info->af_file, AF_MD5, NULL, buf, &buf_len) == 0) {
         int i;
@@ -219,44 +211,84 @@ aff_imgstat(TSK_IMG_INFO * img_info, FILE * hFile)
     return;
 }
 
-void
+static void
 aff_close(TSK_IMG_INFO * img_info)
 {
     IMG_AFF_INFO *aff_info = (IMG_AFF_INFO *) img_info;
     af_close(aff_info->af_file);
-    free(aff_info);
+    tsk_img_free(aff_info);
 }
 
 
 TSK_IMG_INFO *
-aff_open(const char **images, TSK_IMG_INFO * next)
+aff_open(const TSK_TCHAR * const images[], unsigned int a_ssize)
 {
     IMG_AFF_INFO *aff_info;
     TSK_IMG_INFO *img_info;
     int type;
+    char *image;
 
-    if (next != NULL) {
+#ifdef TSK_WIN32
+    // convert wchar_t* image path to char* to conform to
+    // the AFFLIB API
+    UTF16 *utf16 = (UTF16 *) images[0];
+    size_t ilen = wcslen(utf16);
+    size_t olen = ilen * 4 + 1;
+    UTF8 *utf8 = (UTF8 *) tsk_malloc(olen);
+
+    image = (char *) utf8;
+    if (image == NULL)
+        return NULL;
+    TSKConversionResult retval =
+        tsk_UTF16toUTF8_lclorder((const UTF16 **) &utf16,
+        &utf16[ilen], &utf8,
+        &utf8[olen], TSKlenientConversion);
+    *utf8 = '\0';
+    if (retval != TSKconversionOK) {
         tsk_error_reset();
-        tsk_errno = TSK_ERR_IMG_LAYERS;
-        snprintf(tsk_errstr, TSK_ERRSTR_L, "AFF must be lowest layer");
+        tsk_error_set_errno(TSK_ERR_FS_UNICODE);
+        tsk_error_set_errstr("aff_open file: %" PRIttocTSK
+            ": Error converting path to UTF-8 %d\n", images[0], retval);
+        free(image);
         return NULL;
     }
+    utf8 = (UTF8 *) image;
+    while (*utf8) {
+        if (*utf8 > 127) {
+            tsk_error_reset();
+            tsk_error_set_errno(TSK_ERR_FS_UNICODE);
+            tsk_error_set_errstr("aff_open file: %" PRIttocTSK
+                ": Non-Latin paths are not supported for AFF images\n",
+                images[0]);
+            free(image);
+            return NULL;
+        }
+        utf8++;
+    }
+#else
+    image = (char *) tsk_malloc(strlen(images[0]) + 1);
+    if (image == NULL)
+        return NULL;
+    strncpy(image, images[0], strlen(images[0]) + 1);
+#endif
 
-    aff_info = (IMG_AFF_INFO *) tsk_malloc(sizeof(IMG_AFF_INFO));
-    if (aff_info == NULL) {
+    if ((aff_info =
+            (IMG_AFF_INFO *) tsk_img_malloc(sizeof(IMG_AFF_INFO))) ==
+        NULL) {
+        free(image);
         return NULL;
     }
-    memset((void *) aff_info, 0, sizeof(IMG_AFF_INFO));
 
     img_info = (TSK_IMG_INFO *) aff_info;
-
-    img_info->read_random = aff_read_random;
-    img_info->get_size = aff_get_size;
+    img_info->read = aff_read;
     img_info->close = aff_close;
     img_info->imgstat = aff_imgstat;
 
+    img_info->sector_size = 512;
+    if (a_ssize)
+        img_info->sector_size = a_ssize;
 
-    type = af_identify_file_type(images[0], 1);
+    type = af_identify_file_type(image, 1);
     if ((type == AF_IDENTIFY_ERR) || (type == AF_IDENTIFY_NOEXIST)) {
         if (tsk_verbose) {
             tsk_fprintf(stderr,
@@ -265,58 +297,62 @@ aff_open(const char **images, TSK_IMG_INFO * next)
             perror("aff_open");
         }
         tsk_error_reset();
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "aff_open file: %" PRIttocTSK ": Error checking type",
-            images[0]);
-        tsk_errstr2[0] = '\0';
-        free(aff_info);
+        tsk_error_set_errno(TSK_ERR_IMG_OPEN);
+        tsk_error_set_errstr("aff_open file: %" PRIttocTSK
+            ": Error checking type", images[0]);
+        tsk_img_free(aff_info);
+        free(image);
         return NULL;
     }
     else if (type == AF_IDENTIFY_AFF) {
-        img_info->itype = TSK_IMG_INFO_TYPE_AFF_AFF;
+        img_info->itype = TSK_IMG_TYPE_AFF_AFF;
     }
     else if (type == AF_IDENTIFY_AFD) {
-        img_info->itype = TSK_IMG_INFO_TYPE_AFF_AFD;
+        img_info->itype = TSK_IMG_TYPE_AFF_AFD;
     }
     else if (type == AF_IDENTIFY_AFM) {
-        img_info->itype = TSK_IMG_INFO_TYPE_AFF_AFM;
+        img_info->itype = TSK_IMG_TYPE_AFF_AFM;
     }
-//    else if ((type == AF_IDENTIFY_EVF) || (type ==AF_IDENTIFY_EVD  )) {
-//      img_info->itype = TSK_IMG_INFO_TYPE_AFF_AFF;
-    //   }
     else {
-        tsk_error_reset();
-        tsk_errno = TSK_ERR_IMG_MAGIC;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "aff_open: Not an AFF, AFD, or AFM file");
-        free(aff_info);
-        if (tsk_verbose)
-            tsk_fprintf(stderr, "Not an AFF/AFD/AFM file\n");
-
-        return NULL;
+        img_info->itype = TSK_IMG_TYPE_AFF_ANY;
     }
 
-    aff_info->af_file = af_open(images[0], O_RDONLY, 0);
+    aff_info->af_file = af_open(image, O_RDONLY | O_BINARY, 0);
     if (!aff_info->af_file) {
+        // @@@ Need to check here if the open failed because of an incorrect password. 
         tsk_error_reset();
-        tsk_errno = TSK_ERR_IMG_OPEN;
-        snprintf(tsk_errstr, TSK_ERRSTR_L,
-            "aff_open file: %" PRIttocTSK ": Error opening - %s",
-            images[0], strerror(errno));
-        free(aff_info);
+        tsk_error_set_errno(TSK_ERR_IMG_OPEN);
+        tsk_error_set_errstr("aff_open file: %" PRIttocTSK
+            ": Error opening - %s", images[0], strerror(errno));
+        tsk_img_free(aff_info);
         if (tsk_verbose) {
             tsk_fprintf(stderr, "Error opening AFF/AFD/AFM file\n");
             perror("aff_open");
         }
+        free(image);
         return NULL;
     }
+    // verify that a password was given and we can read encrypted data. 
+    if (af_cannot_decrypt(aff_info->af_file)) {
+        tsk_error_reset();
+        tsk_error_set_errno(TSK_ERR_IMG_PASSWD);
+        tsk_error_set_errstr("aff_open file: %" PRIttocTSK, images[0]);
+        tsk_img_free(aff_info);
+        if (tsk_verbose) {
+            tsk_fprintf(stderr,
+                "Error opening AFF/AFD/AFM file (incorrect password)\n");
+        }
+        free(image);
+        return NULL;
+    }
+
     aff_info->type = type;
 
     img_info->size = af_imagesize(aff_info->af_file);
 
     af_seek(aff_info->af_file, 0, SEEK_SET);
     aff_info->seek_pos = 0;
-
+    free(image);
     return img_info;
 }
 #endif
